@@ -7,23 +7,7 @@
  * Copyright (c) 2008, Frank Warmerdam
  * Copyright (c) 2009-2020, Even Rouault <even dot rouault at spatialys.com>
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #include "cpl_port.h"
@@ -55,8 +39,6 @@
 
 using namespace gdal::polygonizer;
 
-CPL_CVSID("$Id$")
-
 /************************************************************************/
 /*                          GPMaskImageData()                           */
 /*                                                                      */
@@ -338,17 +320,29 @@ static CPLErr GDALPolygonizeT(GDALRasterBandH hSrcBand,
                         : oFirstEnum.panPolyIdMap[panThisLineId[iX]];
             }
 
-            oPolygonizer.processLine(panLastLineId, panLastLineVal,
-                                     paoThisLineArm, paoLastLineArm, iY,
-                                     nXSize);
-            eErr = oPolygonWriter.getErr();
+            if (!oPolygonizer.processLine(panLastLineId, panLastLineVal,
+                                          paoThisLineArm, paoLastLineArm, iY,
+                                          nXSize))
+            {
+                eErr = CE_Failure;
+            }
+            else
+            {
+                eErr = oPolygonWriter.getErr();
+            }
         }
         else
         {
-            oPolygonizer.processLine(panThisLineId, panLastLineVal,
-                                     paoThisLineArm, paoLastLineArm, iY,
-                                     nXSize);
-            eErr = oPolygonWriter.getErr();
+            if (!oPolygonizer.processLine(panThisLineId, panLastLineVal,
+                                          paoThisLineArm, paoLastLineArm, iY,
+                                          nXSize))
+            {
+                eErr = CE_Failure;
+            }
+            else
+            {
+                eErr = oPolygonWriter.getErr();
+            }
         }
 
         if (eErr != CE_None)
