@@ -7,23 +7,7 @@
  ******************************************************************************
  * Copyright (c) 2007, Howard Butler
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #include "ogrsf_frmts.h"
@@ -163,7 +147,7 @@ MAIN_START(nArgc, papszArgv)
                                     "Z"))
             {
                 bIs3D = TRUE;
-                osGeomName.resize(osGeomName.size() - 1);
+                osGeomName.pop_back();
             }
             if (EQUAL(osGeomName, "NONE"))
                 eGType = wkbNone;
@@ -462,6 +446,7 @@ MAIN_START(nArgc, papszArgv)
 
     return 0;
 }
+
 MAIN_END
 
 /************************************************************************/
@@ -709,6 +694,7 @@ GeometriesList *FlattenGeometries(GeometriesList *input)
 
     return output;
 }
+
 /************************************************************************/
 /*                           DissolveLayer()                            */
 /************************************************************************/
