@@ -7,28 +7,13 @@
  ******************************************************************************
  * Copyright (c) 2022, Even Rouault <even dot rouault at spatialys.com>
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #include "gdal_pam.h"
 #include "common.h"
 #include "include_basisu_sdk.h"
+#include "basisudrivercore.h"
 
 #include <algorithm>
 #include <cstdlib>
@@ -63,7 +48,7 @@ class BASISUDataset final : public GDALPamDataset
     BASISUDataset(uint32_t iImageIdx, void *pEncodedData,
                   uint32_t nEncodedDataSize);
     BASISUDataset(BASISUDataset *poParent, uint32_t iLevel);
-    static int Identify(GDALOpenInfo *poOpenInfo);
+
     static GDALDataset *Open(GDALOpenInfo *poOpenInfo);
     static GDALDataset *CreateCopy(const char *pszFilename,
                                    GDALDataset *poSrcDS, int bStrict,
@@ -88,33 +73,6 @@ class BASISURasterBand final : public GDALPamRasterBand
     GDALRasterBand *GetOverview(int nIdx) override;
 };
 
-/************************************************************************/
-/*                                Identify()                            */
-/************************************************************************/
-
-int BASISUDataset::Identify(GDALOpenInfo *poOpenInfo)
-{
-    if (STARTS_WITH(poOpenInfo->pszFilename, "BASISU:"))
-        return true;
-    // See
-    // https://github.com/BinomialLLC/basis_universal/blob/master/spec/basis_spec.txt
-    constexpr int HEADER_SIZE = 77;
-    if (!(poOpenInfo->fpL != nullptr &&
-          poOpenInfo->nHeaderBytes >= HEADER_SIZE &&
-          poOpenInfo->pabyHeader[0] == 0x73 &&         // Signature
-          poOpenInfo->pabyHeader[1] == 0x42 &&         // Signature
-          poOpenInfo->pabyHeader[4] == HEADER_SIZE &&  // Header size
-          poOpenInfo->pabyHeader[5] == 0))             // Header size
-    {
-        return false;
-    }
-    const uint32_t nDataSize = CPL_LSBUINT32PTR(poOpenInfo->pabyHeader + 8);
-    VSIFSeekL(poOpenInfo->fpL, 0, SEEK_END);
-    const auto nFileSize = VSIFTellL(poOpenInfo->fpL);
-    VSIFSeekL(poOpenInfo->fpL, 0, SEEK_SET);
-    return nDataSize + HEADER_SIZE == nFileSize;
-}
-
 /************************************************************************/
 /*                           BASISUDataset()                            */
 /************************************************************************/
@@ -270,7 +228,7 @@ GDALRasterBand *BASISURasterBand::GetOverview(int nIdx)
 
 GDALDataset *BASISUDataset::Open(GDALOpenInfo *poOpenInfo)
 {
-    if (!Identify(poOpenInfo) || poOpenInfo->eAccess == GA_Update)
+    if (!BASISUDriverIdentify(poOpenInfo) || poOpenInfo->eAccess == GA_Update)
         return nullptr;
 
     VSILFILE *fpL = nullptr;
@@ -306,7 +264,7 @@ GDALDataset *BASISUDataset::Open(GDALOpenInfo *poOpenInfo)
     }
     const uint32_t nSize = static_cast<uint32_t>(nSizeLarge);
 
-    auto poDS = cpl::make_unique<BASISUDataset>(
+    auto poDS = std::make_unique<BASISUDataset>(
         nImageIdx != static_cast<uint32_t>(-1) ? nImageIdx : 0, pabyRet, nSize);
     auto &transcoder = poDS->m_transcoder;
     basist::basisu_file_info file_info;
@@ -376,7 +334,7 @@ GDALDataset *BASISUDataset::Open(GDALOpenInfo *poOpenInfo)
                                             poDS->m_iImageIdx, level_index))
         {
             auto poOverviewDS =
-                cpl::make_unique<BASISUDataset>(poDS.get(), level_index);
+                std::make_unique<BASISUDataset>(poDS.get(), level_index);
             for (int i = 1; i <= l_nBands; ++i)
             {
                 poOverviewDS->SetBand(
@@ -421,26 +379,12 @@ GDALDataset *BASISUDataset::CreateCopy(const char *pszFilename,
 
 void GDALRegister_BASISU()
 {
-    if (GDALGetDriverByName("BASISU") != nullptr)
+    if (GDALGetDriverByName(BASISU_DRIVER_NAME) != nullptr)
         return;
 
     GDALDriver *poDriver = new GDALDriver();
+    BASISUDriverSetCommonMetadata(poDriver);
 
-    poDriver->SetDescription("BASISU");
-    poDriver->SetMetadataItem(GDAL_DCAP_RASTER, "YES");
-    poDriver->SetMetadataItem(GDAL_DMD_LONGNAME,
-                              "Basis Universal texture format");
-    poDriver->SetMetadataItem(GDAL_DMD_HELPTOPIC, "drivers/raster/basisu.html");
-    poDriver->SetMetadataItem(GDAL_DMD_EXTENSION, "basis");
-    poDriver->SetMetadataItem(GDAL_DMD_CREATIONDATATYPES, "Byte");
-
-    poDriver->SetMetadataItem(GDAL_DCAP_VIRTUALIO, "YES");
-
-    poDriver->SetMetadataItem(
-        GDAL_DMD_CREATIONOPTIONLIST,
-        GDAL_KTX2_BASISU_GetCreationOptions(/* bIsKTX2 = */ false).c_str());
-
-    poDriver->pfnIdentify = BASISUDataset::Identify;
     poDriver->pfnOpen = BASISUDataset::Open;
     poDriver->pfnCreateCopy = BASISUDataset::CreateCopy;
 
