@@ -7,23 +7,7 @@
  ******************************************************************************
  * Copyright (c) 2019, Even Rouault <even.rouault at spatialys.com>
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 // Example product:
@@ -35,8 +19,6 @@
 
 #include <limits>
 
-#ifdef NETCDF_HAS_NC4
-
 /************************************************************************/
 /*                      Sentinel3_SRAL_MWR_Layer                        */
 /************************************************************************/
@@ -48,6 +30,7 @@ class Sentinel3_SRAL_MWR_Layer final : public OGRLayer
     size_t m_nCurIdx = 0;
     size_t m_nFeatureCount = 0;
     CPLStringList m_aosMetadata{};
+
     struct VariableInfo
     {
         int varid;
@@ -56,6 +39,7 @@ class Sentinel3_SRAL_MWR_Layer final : public OGRLayer
         double offset;
         double nodata;
     };
+
     std::vector<VariableInfo> m_asVarInfo{};
     int m_iLongitude = -1;
     int m_iLatitude = -1;
@@ -75,6 +59,7 @@ class Sentinel3_SRAL_MWR_Layer final : public OGRLayer
     {
         return m_poFDefn;
     }
+
     void ResetReading() override;
     OGRFeature *GetNextFeature() override;
     OGRFeature *GetFeature(GIntBig nFID) override;
@@ -490,5 +475,3 @@ void netCDFDataset::ProcessSentinel3_SRAL_MWR()
     if (poSRS)
         poSRS->Release();
 }
-
-#endif  // NETCDF_HAS_NC4
