@@ -8,23 +8,7 @@
  ******************************************************************************
  * Copyright (c) 2004, Frank Warmerdam
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #ifndef NETCDFDATASET_H_INCLUDED_
@@ -144,9 +128,9 @@ static const int NCDF_DEFLATE_LEVEL = 1; /* best time/size ratio */
         if (NCDF_ERR_status_ != NC_NOERR)                                      \
         {                                                                      \
             CPLError(CE_Failure, CPLE_AppDefined,                              \
-                     "netcdf error #%d : %s .\nat (%s,%s,%d)\n", status,       \
-                     nc_strerror(NCDF_ERR_status_), __FILE__, __FUNCTION__,    \
-                     __LINE__);                                                \
+                     "netcdf error #%d : %s .\nat (%s,%s,%d)\n",               \
+                     NCDF_ERR_status_, nc_strerror(NCDF_ERR_status_),          \
+                     __FILE__, __FUNCTION__, __LINE__);                        \
         }                                                                      \
     } while (0)
 
@@ -358,6 +342,7 @@ class netCDFDataset final : public GDALPamDataset
     friend class netCDFRasterBand;  // TMP
     friend class netCDFLayer;
     friend class netCDFVariable;
+    friend class nccfdriver::netCDFVID;
 
     typedef enum
     {
@@ -565,8 +550,6 @@ class netCDFDataset final : public GDALPamDataset
     void SetSpatialRefNoUpdate(const OGRSpatialReference *);
 
   protected:
-    CPLXMLNode *SerializeToXML(const char *pszVRTPath) override;
-
     OGRLayer *ICreateLayer(const char *pszName,
                            const OGRGeomFieldDefn *poGeomFieldDefn,
                            CSLConstList papszOptions) override;
@@ -713,7 +696,7 @@ class netCDFLayer final : public OGRLayer
 
     OGRFeature *GetNextRawFeature();
     double Get1DVarAsDouble(int nVarId, nc_type nVarType, size_t nIndex,
-                            NCDFNoDataUnion noDataVal, bool *pbIsNoData);
+                            const NCDFNoDataUnion &noDataVal, bool *pbIsNoData);
     CPLErr GetFillValue(int nVarID, char **ppszValue);
     CPLErr GetFillValue(int nVarID, double *pdfValue);
     void GetNoDataValueForFloat(int nVarId, NCDFNoDataUnion *puNoData);
