@@ -9,23 +9,7 @@
  * Copyright (c) 2002, Frank Warmerdam
  * Copyright (c) 2009-2010, Even Rouault <even dot rouault at spatialys.com>
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #include "nitflib.h"
@@ -628,24 +612,26 @@ int main(int nArgc, char **papszArgv)
                 EQUAL(CSLFetchNameValueDef(psDES->papszMetadata, "DESID", ""),
                       "CSSHPA DES"))
             {
-                char szFilename[40];
-                char szRadix[32];
+                char szFilename[512];
+                char szRadix[256];
                 if (bExtractSHPInMem)
-                    snprintf(szRadix, sizeof(szRadix),
-                             "/vsimem/nitf_segment_%d", iSegment + 1);
+                    snprintf(szRadix, sizeof(szRadix), "%s",
+                             VSIMemGenerateHiddenFilename(
+                                 CPLSPrintf("nitf_segment_%d", iSegment + 1)));
                 else
                     snprintf(szRadix, sizeof(szRadix), "nitf_segment_%d",
                              iSegment + 1);
 
                 if (NITFDESExtractShapefile(psDES, szRadix))
                 {
-                    OGRDataSourceH hDS;
+                    GDALDatasetH hDS;
                     snprintf(szFilename, sizeof(szFilename), "%s.SHP", szRadix);
-                    hDS = OGROpen(szFilename, FALSE, NULL);
+                    hDS = GDALOpenEx(szFilename, GDAL_OF_VECTOR, NULL, NULL,
+                                     NULL);
                     if (hDS)
                     {
                         int nGeom = 0;
-                        OGRLayerH hLayer = OGR_DS_GetLayer(hDS, 0);
+                        OGRLayerH hLayer = GDALDatasetGetLayer(hDS, 0);
                         if (hLayer)
                         {
                             OGRFeatureH hFeat;
@@ -667,7 +653,7 @@ int main(int nArgc, char **papszArgv)
                                 OGR_F_Destroy(hFeat);
                             }
                         }
-                        OGR_DS_Destroy(hDS);
+                        GDALClose(hDS);
                     }
                 }
 
