@@ -7,23 +7,7 @@
  ******************************************************************************
  * Copyright (c) 2007-2014, Even Rouault <even dot rouault at spatialys.com>
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #include "cpl_port.h"
@@ -46,6 +30,7 @@
 #include "ogr_spatialref.h"
 #include "nitflib.h"
 #include "vrtdataset.h"
+#include "nitfdrivercore.h"
 
 constexpr int GEOTRSFRM_TOPLEFT_X = 0;
 constexpr int GEOTRSFRM_WE_RES = 1;
@@ -140,13 +125,10 @@ class RPFTOCDataset final : public GDALPamDataset
     }
 
     static int IsNITFFileTOC(NITFFile *psFile);
-    static int IsNonNITFFileTOC(GDALOpenInfo *poOpenInfo,
-                                const char *pszFilename);
     static GDALDataset *OpenFileTOC(NITFFile *psFile, const char *pszFilename,
                                     const char *entryName,
                                     const char *openInformationName);
 
-    static int Identify(GDALOpenInfo *poOpenInfo);
     static GDALDataset *Open(GDALOpenInfo *poOpenInfo);
 };
 
@@ -321,6 +303,7 @@ class RPFTOCProxyRasterBandRGBA final : public GDALPamRasterBand
         blockByteSize = nBlockXSize * nBlockYSize;
         memset(colorTable, 0, sizeof(colorTable));
     }
+
     virtual ~RPFTOCProxyRasterBandRGBA()
     {
     }
@@ -1075,38 +1058,6 @@ GDALDataset *RPFTOCSubDataset::CreateDataSetFromTocEntry(
     return poVirtualDS;
 }
 
-/************************************************************************/
-/*                          IsNonNITFFileTOC()                          */
-/************************************************************************/
-
-/* Check whether the file is a TOC file without NITF header */
-int RPFTOCDataset::IsNonNITFFileTOC(GDALOpenInfo *poOpenInfo,
-                                    const char *pszFilename)
-{
-    const char pattern[] = {0,   0,   '0', ' ', ' ', ' ', ' ', ' ',
-                            ' ', ' ', 'A', '.', 'T', 'O', 'C'};
-    if (poOpenInfo)
-    {
-        if (poOpenInfo->nHeaderBytes < 48)
-            return FALSE;
-        return memcmp(pattern, poOpenInfo->pabyHeader, 15) == 0;
-    }
-    else
-    {
-        VSILFILE *fp = VSIFOpenL(pszFilename, "rb");
-        if (fp == nullptr)
-        {
-            return FALSE;
-        }
-
-        char buffer[48];
-        int ret = (VSIFReadL(buffer, 1, 48, fp) == 48) &&
-                  memcmp(pattern, buffer, 15) == 0;
-        CPL_IGNORE_RET_VAL(VSIFCloseL(fp));
-        return ret;
-    }
-}
-
 /************************************************************************/
 /*                             IsNITFFileTOC()                          */
 /************************************************************************/
@@ -1293,49 +1244,6 @@ GDALDataset *RPFTOCDataset::OpenFileTOC(NITFFile *psFile,
     return nullptr;
 }
 
-/************************************************************************/
-/*                              Identify()                              */
-/************************************************************************/
-
-int RPFTOCDataset::Identify(GDALOpenInfo *poOpenInfo)
-
-{
-    const char *pszFilename = poOpenInfo->pszFilename;
-
-    /* -------------------------------------------------------------------- */
-    /*      Is this a sub-dataset selector? If so, it is obviously RPFTOC.  */
-    /* -------------------------------------------------------------------- */
-
-    if (STARTS_WITH_CI(pszFilename, "NITF_TOC_ENTRY:"))
-        return TRUE;
-
-    /* -------------------------------------------------------------------- */
-    /*      First we check to see if the file has the expected header       */
-    /*      bytes.                                                          */
-    /* -------------------------------------------------------------------- */
-    if (poOpenInfo->nHeaderBytes < 48)
-        return FALSE;
-
-    if (IsNonNITFFileTOC(poOpenInfo, pszFilename))
-        return TRUE;
-
-    if (!STARTS_WITH_CI((char *)poOpenInfo->pabyHeader, "NITF") &&
-        !STARTS_WITH_CI((char *)poOpenInfo->pabyHeader, "NSIF") &&
-        !STARTS_WITH_CI((char *)poOpenInfo->pabyHeader, "NITF"))
-        return FALSE;
-
-    /* If it is a NITF A.TOC file, it must contain A.TOC in its header */
-    for (int i = 0; i < static_cast<int>(poOpenInfo->nHeaderBytes) -
-                            static_cast<int>(strlen("A.TOC"));
-         i++)
-    {
-        if (STARTS_WITH_CI((const char *)poOpenInfo->pabyHeader + i, "A.TOC"))
-            return TRUE;
-    }
-
-    return FALSE;
-}
-
 /************************************************************************/
 /*                                Open()                                */
 /************************************************************************/
@@ -1343,7 +1251,7 @@ int RPFTOCDataset::Identify(GDALOpenInfo *poOpenInfo)
 GDALDataset *RPFTOCDataset::Open(GDALOpenInfo *poOpenInfo)
 
 {
-    if (!Identify(poOpenInfo))
+    if (!RPFTOCDriverIdentify(poOpenInfo))
         return nullptr;
 
     const char *pszFilename = poOpenInfo->pszFilename;
@@ -1368,8 +1276,8 @@ GDALDataset *RPFTOCDataset::Open(GDALOpenInfo *poOpenInfo)
         pszFilename++;
     }
 
-    if (IsNonNITFFileTOC((entryName != nullptr) ? nullptr : poOpenInfo,
-                         pszFilename))
+    if (RPFTOCIsNonNITFFileTOC((entryName != nullptr) ? nullptr : poOpenInfo,
+                               pszFilename))
     {
         GDALDataset *poDS = OpenFileTOC(nullptr, pszFilename, entryName,
                                         poOpenInfo->pszFilename);
@@ -1434,23 +1342,13 @@ GDALDataset *RPFTOCDataset::Open(GDALOpenInfo *poOpenInfo)
 void GDALRegister_RPFTOC()
 
 {
-    if (GDALGetDriverByName("RPFTOC") != nullptr)
+    if (GDALGetDriverByName(RPFTOC_DRIVER_NAME) != nullptr)
         return;
 
     GDALDriver *poDriver = new GDALDriver();
+    RPFTOCDriverSetCommonMetadata(poDriver);
 
-    poDriver->SetDescription("RPFTOC");
-    poDriver->SetMetadataItem(GDAL_DCAP_RASTER, "YES");
-    poDriver->SetMetadataItem(GDAL_DMD_LONGNAME,
-                              "Raster Product Format TOC format");
-
-    poDriver->pfnIdentify = RPFTOCDataset::Identify;
     poDriver->pfnOpen = RPFTOCDataset::Open;
 
-    poDriver->SetMetadataItem(GDAL_DMD_HELPTOPIC, "drivers/raster/rpftoc.html");
-    poDriver->SetMetadataItem(GDAL_DMD_EXTENSION, "toc");
-    poDriver->SetMetadataItem(GDAL_DCAP_VIRTUALIO, "YES");
-    poDriver->SetMetadataItem(GDAL_DMD_SUBDATASETS, "YES");
-
     GetGDALDriverManager()->RegisterDriver(poDriver);
 }
