@@ -8,23 +8,7 @@
  * Copyright (c) 2007, Waypoint Information Technology
  * Copyright (c) 2009-2011, Even Rouault <even dot rouault at spatialys.com>
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #include "gdal_pam.h"
@@ -32,6 +16,7 @@
 #include "northwood.h"
 
 #include <algorithm>
+#include <cassert>
 #include <limits>
 #include <string>
 
@@ -79,6 +64,7 @@ int nwt_ParseHeader(NWT_GRID *pGrd, const unsigned char *nwtHeader)
     memcpy(&pGrd->dfMaxY, &nwtHeader[37], sizeof(pGrd->dfMaxY));
     CPL_LSBPTR64(&pGrd->dfMaxY);
 
+    assert(pGrd->nXSide > 1);
     pGrd->dfStepSize = (pGrd->dfMaxX - pGrd->dfMinX) / (pGrd->nXSide - 1);
     /* dfTmp = (pGrd->dfMaxY - pGrd->dfMinY) / (pGrd->nYSide - 1); */
 
@@ -184,14 +170,14 @@ int nwt_ParseHeader(NWT_GRID *pGrd, const unsigned char *nwtHeader)
         }
         CPL_LSBPTR16(&usTmp);
         pGrd->stClassDict = reinterpret_cast<NWT_CLASSIFIED_DICT *>(
-            calloc(sizeof(NWT_CLASSIFIED_DICT), 1));
+            calloc(1, sizeof(NWT_CLASSIFIED_DICT)));
 
         pGrd->stClassDict->nNumClassifiedItems = usTmp;
 
         pGrd->stClassDict->stClassifiedItem =
             reinterpret_cast<NWT_CLASSIFIED_ITEM **>(
-                calloc(sizeof(NWT_CLASSIFIED_ITEM *),
-                       pGrd->stClassDict->nNumClassifiedItems + 1));
+                calloc(pGrd->stClassDict->nNumClassifiedItems + 1,
+                       sizeof(NWT_CLASSIFIED_ITEM *)));
 
         // load the dictionary
         for (unsigned int iItem = 0;
@@ -200,7 +186,7 @@ int nwt_ParseHeader(NWT_GRID *pGrd, const unsigned char *nwtHeader)
             NWT_CLASSIFIED_ITEM *psItem =
                 pGrd->stClassDict->stClassifiedItem[iItem] =
                     reinterpret_cast<NWT_CLASSIFIED_ITEM *>(
-                        calloc(sizeof(NWT_CLASSIFIED_ITEM), 1));
+                        calloc(1, sizeof(NWT_CLASSIFIED_ITEM)));
 
             unsigned char cTmp[256];
             if (!VSIFReadL(&cTmp, 9, 1, pGrd->fp))
@@ -424,7 +410,7 @@ NWT_GRID *nwtOpenGrid(char *filename)
         nwtHeader[3] != 'C')
         return nullptr;
 
-    NWT_GRID *pGrd = reinterpret_cast<NWT_GRID *>(calloc(sizeof(NWT_GRID), 1));
+    NWT_GRID *pGrd = reinterpret_cast<NWT_GRID *>(calloc(1, sizeof(NWT_GRID)));
 
     if (nwtHeader[4] == '1')
         pGrd->cFormat = 0x00;  // grd - surface type
