@@ -7,23 +7,7 @@
  ******************************************************************************
  * Copyright (c) PCRaster owners
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #include "cpl_string.h"
@@ -32,6 +16,7 @@
 #include "pcrasterrasterband.h"
 #include "pcrasterdataset.h"
 #include "pcrasterutil.h"
+#include "pcrasterdrivercore.h"
 
 /*!
   \file
@@ -53,9 +38,7 @@ GDALDataset *PCRasterDataset::open(GDALOpenInfo *info)
 {
     PCRasterDataset *dataset = nullptr;
 
-    if (info->fpL && info->nHeaderBytes >= static_cast<int>(CSF_SIZE_SIG) &&
-        strncmp(reinterpret_cast<char *>(info->pabyHeader), CSF_SIG,
-                CSF_SIZE_SIG) == 0)
+    if (PCRasterDriverIdentify(info))
     {
         MOPEN_PERM mode = info->eAccess == GA_Update ? M_READ_WRITE : M_READ;
 
