@@ -8,23 +8,7 @@
  * Copyright (c) 1999, Frank Warmerdam
  * Copyright (c) 2008-2010, Even Rouault <even dot rouault at spatialys.com>
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #include "cpl_string.h"
@@ -75,14 +59,17 @@ class PAuxDataset final : public RawDataset
     {
         return m_oSRS.IsEmpty() ? nullptr : &m_oSRS;
     }
+
     CPLErr GetGeoTransform(double *) override;
     CPLErr SetGeoTransform(double *) override;
 
     int GetGCPCount() override;
+
     const OGRSpatialReference *GetGCPSpatialRef() const override
     {
         return m_oGCPSRS.IsEmpty() ? nullptr : &m_oGCPSRS;
     }
+
     const GDAL_GCP *GetGCPs() override;
 
     char **GetFileList() override;
@@ -679,7 +666,7 @@ GDALDataset *PAuxDataset::Open(GDALOpenInfo *poOpenInfo)
     /* -------------------------------------------------------------------- */
     /*      Create a corresponding GDALDataset.                             */
     /* -------------------------------------------------------------------- */
-    auto poDS = cpl::make_unique<PAuxDataset>();
+    auto poDS = std::make_unique<PAuxDataset>();
 
     /* -------------------------------------------------------------------- */
     /*      Load the .aux file into a string list suitable to be            */
@@ -801,7 +788,7 @@ GDALDataset *PAuxDataset::Open(GDALOpenInfo *poOpenInfo)
             continue;
         }
 
-        auto poBand = cpl::make_unique<PAuxRasterBand>(
+        auto poBand = std::make_unique<PAuxRasterBand>(
             poDS.get(), poDS->nBands + 1, poDS->fpImage, nBandOffset,
             nPixelOffset, nLineOffset, eType, bNative);
         if (!poBand->IsValid())
@@ -962,7 +949,8 @@ GDALDataset *PAuxDataset::Create(const char *pszFilename, int nXSize,
         {
             nPixelOffset = GDALGetDataTypeSizeBytes(eType);
             nLineOffset = nXSize * nPixelSizeSum;
-            nNextImgOffset = nImgOffset + nPixelOffset * nXSize;
+            nNextImgOffset =
+                nImgOffset + static_cast<vsi_l_offset>(nPixelOffset) * nXSize;
         }
         else if (EQUAL(pszInterleave, "PIXEL"))
         {
