@@ -11,23 +11,7 @@
  * Copyright (c) 2005, Frank Warmerdam <warmerdam@pobox.com>
  * Copyright (c) 2007-2011, Even Rouault <even dot rouault at spatialys.com>
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #include "cpl_port.h"
@@ -57,6 +41,8 @@ class SRTMHGTDataset final : public GDALPamDataset
     GByte *pabyBuffer = nullptr;
     OGRSpatialReference m_oSRS{};
 
+    static GDALPamDataset *OpenPAM(GDALOpenInfo *);
+
   public:
     SRTMHGTDataset();
     virtual ~SRTMHGTDataset();
@@ -344,11 +330,12 @@ int SRTMHGTDataset::Identify(GDALOpenInfo *poOpenInfo)
 
     if (VSIStatL(poOpenInfo->pszFilename, &fileStat) != 0)
         return FALSE;
-    if (fileStat.st_size != 3601 * 3601 &&
+    if (fileStat.st_size != 1201 * 1201 * 2 &&
+        fileStat.st_size != 1801 * 3601 * 2 &&
+        fileStat.st_size != 3601 * 3601 &&
         fileStat.st_size != 3601 * 3601 * 2 &&
         fileStat.st_size != 3601 * 3601 * 4 &&  // .hgts
-        fileStat.st_size != 1801 * 3601 * 2 &&
-        fileStat.st_size != 1201 * 1201 * 2)
+        fileStat.st_size != 7201 * 7201 * 2)
         return FALSE;
 
     return TRUE;
@@ -359,6 +346,15 @@ int SRTMHGTDataset::Identify(GDALOpenInfo *poOpenInfo)
 /************************************************************************/
 
 GDALDataset *SRTMHGTDataset::Open(GDALOpenInfo *poOpenInfo)
+{
+    return OpenPAM(poOpenInfo);
+}
+
+/************************************************************************/
+/*                              OpenPAM()                               */
+/************************************************************************/
+
+GDALPamDataset *SRTMHGTDataset::OpenPAM(GDALOpenInfo *poOpenInfo)
 {
     if (!Identify(poOpenInfo))
         return nullptr;
@@ -373,7 +369,7 @@ GDALDataset *SRTMHGTDataset::Open(GDALOpenInfo *poOpenInfo)
         osFilename += CPLString(fileName).substr(0, 7);
         osFilename += ".hgt";
         GDALOpenInfo oOpenInfo(osFilename, poOpenInfo->eAccess);
-        GDALDataset *poDS = Open(&oOpenInfo);
+        auto poDS = OpenPAM(&oOpenInfo);
         if (poDS != nullptr)
         {
             // override description with the main one
@@ -391,7 +387,7 @@ GDALDataset *SRTMHGTDataset::Open(GDALOpenInfo *poOpenInfo)
         osFilename += CPLString(fileName).substr(0, 7);
         osFilename += ".raw";
         GDALOpenInfo oOpenInfo(osFilename, poOpenInfo->eAccess);
-        GDALDataset *poDS = Open(&oOpenInfo);
+        auto poDS = OpenPAM(&oOpenInfo);
         if (poDS != nullptr)
         {
             // override description with the main one
@@ -426,7 +422,7 @@ GDALDataset *SRTMHGTDataset::Open(GDALOpenInfo *poOpenInfo)
     /* -------------------------------------------------------------------- */
     /*      Create a corresponding GDALDataset.                             */
     /* -------------------------------------------------------------------- */
-    SRTMHGTDataset *poDS = new SRTMHGTDataset();
+    auto poDS = std::make_unique<SRTMHGTDataset>();
 
     poDS->fpImage = poOpenInfo->fpL;
     poOpenInfo->fpL = nullptr;
@@ -434,7 +430,6 @@ GDALDataset *SRTMHGTDataset::Open(GDALOpenInfo *poOpenInfo)
     VSIStatBufL fileStat;
     if (VSIStatL(poOpenInfo->pszFilename, &fileStat) != 0)
     {
-        delete poDS;
         return nullptr;
     }
 
@@ -443,6 +438,13 @@ GDALDataset *SRTMHGTDataset::Open(GDALOpenInfo *poOpenInfo)
     GDALDataType eDT = GDT_Int16;
     switch (fileStat.st_size)
     {
+        case 1201 * 1201 * 2:
+            numPixels_x = numPixels_y = 1201;
+            break;
+        case 1801 * 3601 * 2:
+            numPixels_x = 1801;
+            numPixels_y = 3601;
+            break;
         case 3601 * 3601:
             numPixels_x = numPixels_y = 3601;
             eDT = GDT_Byte;
@@ -454,12 +456,8 @@ GDALDataset *SRTMHGTDataset::Open(GDALOpenInfo *poOpenInfo)
             numPixels_x = numPixels_y = 3601;
             eDT = GDT_Float32;
             break;
-        case 1801 * 3601 * 2:
-            numPixels_x = 1801;
-            numPixels_y = 3601;
-            break;
-        case 1201 * 1201 * 2:
-            numPixels_x = numPixels_y = 1201;
+        case 7201 * 7201 * 2:
+            numPixels_x = numPixels_y = 7201;
             break;
         default:
             numPixels_x = numPixels_y = 0;
@@ -494,7 +492,7 @@ GDALDataset *SRTMHGTDataset::Open(GDALOpenInfo *poOpenInfo)
     /* -------------------------------------------------------------------- */
     /*      Create band information object.                                 */
     /* -------------------------------------------------------------------- */
-    SRTMHGTRasterBand *tmpBand = new SRTMHGTRasterBand(poDS, 1, eDT);
+    SRTMHGTRasterBand *tmpBand = new SRTMHGTRasterBand(poDS.get(), 1, eDT);
     poDS->SetBand(1, tmpBand);
 
     /* -------------------------------------------------------------------- */
@@ -506,9 +504,9 @@ GDALDataset *SRTMHGTDataset::Open(GDALOpenInfo *poOpenInfo)
     /* -------------------------------------------------------------------- */
     /*      Support overviews.                                              */
     /* -------------------------------------------------------------------- */
-    poDS->oOvManager.Initialize(poDS, poOpenInfo->pszFilename);
+    poDS->oOvManager.Initialize(poDS.get(), poOpenInfo->pszFilename);
 
-    return poDS;
+    return poDS.release();
 }
 
 /************************************************************************/
@@ -593,12 +591,13 @@ GDALDataset *SRTMHGTDataset::CreateCopy(const char *pszFilename,
     const int nYSize = poSrcDS->GetRasterYSize();
 
     if (!((nXSize == 1201 && nYSize == 1201) ||
+          (nXSize == 1801 && nYSize == 3601) ||
           (nXSize == 3601 && nYSize == 3601) ||
-          (nXSize == 1801 && nYSize == 3601)))
+          (nXSize == 7201 && nYSize == 7201)))
     {
-        CPLError(
-            CE_Failure, CPLE_AppDefined,
-            "Image dimensions should be 1201x1201, 3601x3601 or 1801x3601.");
+        CPLError(CE_Failure, CPLE_AppDefined,
+                 "Image dimensions should be 1201x1201, 1801x3601, 3601x3601 "
+                 "or 7201x7201.");
         return nullptr;
     }
 
@@ -687,8 +686,8 @@ GDALDataset *SRTMHGTDataset::CreateCopy(const char *pszFilename,
     /* -------------------------------------------------------------------- */
     /*      Reopen and copy missing information into a PAM file.            */
     /* -------------------------------------------------------------------- */
-    GDALPamDataset *poDS =
-        reinterpret_cast<GDALPamDataset *>(GDALOpen(pszFilename, GA_ReadOnly));
+    GDALOpenInfo oOpenInfo(pszFilename, GA_ReadOnly);
+    auto poDS = OpenPAM(&oOpenInfo);
 
     if (poDS)
         poDS->CloneInfo(poSrcDS, GCIF_PAM_DEFAULT);
