@@ -1668,6 +1668,19 @@ void GDALDriverManager::DeclareDeferredPluginDriver(
 
     if (osFullPath.empty())
     {
+        // Do not try to re-register a non-existent deferred plugin
+        // This would cause memory leaks in case of repeated calls to GDALAllRegister()
+        // Cf https://github.com/rasterio/rasterio/issues/3250
+        for (const auto &poDriver : m_aoHiddenDrivers)
+        {
+            if (EQUAL(poDriver->GetDescription(),
+                      poProxyDriver->GetDescription()))
+            {
+                delete poProxyDriver;
+                return;
+            }
+        }
+
         CPLDebug("GDAL",
                  "Proxy driver %s *not* registered due to %s not being found",
                  poProxyDriver->GetDescription(), pszPluginFileName);
