@@ -9,23 +9,7 @@
  ******************************************************************************
  * Copyright (c) 2023, Even Rouault <even.rouault at spatialys.com>
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #include "gdal_alg.h"
@@ -52,7 +36,7 @@ class GDALMDArrayGridded final : public GDALPamMDArray
     std::shared_ptr<GDALMDArray> m_poVarY{};
     std::unique_ptr<GDALDataset> m_poVectorDS{};
     GDALGridAlgorithm m_eAlg;
-    std::unique_ptr<void, CPLFreeReleaser> m_poGridOptions;
+    std::unique_ptr<void, VSIFreeReleaser> m_poGridOptions;
     const GDALExtendedDataType m_dt;
     std::vector<GUInt64> m_anBlockSize{};
     const double m_dfNoDataValue;
@@ -71,7 +55,7 @@ class GDALMDArrayGridded final : public GDALPamMDArray
         const std::shared_ptr<GDALMDArray> &poVarX,
         const std::shared_ptr<GDALMDArray> &poVarY,
         std::unique_ptr<GDALDataset> &&poVectorDS, GDALGridAlgorithm eAlg,
-        std::unique_ptr<void, CPLFreeReleaser> &&poGridOptions,
+        std::unique_ptr<void, VSIFreeReleaser> &&poGridOptions,
         double dfNoDataValue, double dfMinX, double dfResX, double dfMinY,
         double dfResY, double dfRadius)
         : GDALAbstractMDArray(std::string(),
@@ -106,7 +90,7 @@ class GDALMDArrayGridded final : public GDALPamMDArray
            const std::shared_ptr<GDALMDArray> &poVarX,
            const std::shared_ptr<GDALMDArray> &poVarY,
            std::unique_ptr<GDALDataset> &&poVectorDS, GDALGridAlgorithm eAlg,
-           std::unique_ptr<void, CPLFreeReleaser> &&poGridOptions,
+           std::unique_ptr<void, VSIFreeReleaser> &&poGridOptions,
            double dfNoDataValue, double dfMinX, double dfResX, double dfMinY,
            double dfResY, double dfRadius)
     {
@@ -263,7 +247,7 @@ bool GDALMDArrayGridded::IRead(const GUInt64 *arrayStartIdx,
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wnull-dereference"
 #endif
-        m_anLastStartIdx = anStartIdx;
+        m_anLastStartIdx = std::move(anStartIdx);
 #if defined(__GNUC__) && __GNUC__ >= 13
 #pragma GCC diagnostic pop
 #endif
@@ -442,7 +426,7 @@ GDALMDArray::GetGridded(const std::string &osGridOptions,
         return nullptr;
     }
 
-    std::unique_ptr<void, CPLFreeReleaser> poGridOptions(pOptions);
+    std::unique_ptr<void, VSIFreeReleaser> poGridOptions(pOptions);
 
     if (GetDataType().GetClass() != GEDTC_NUMERIC)
     {
@@ -665,9 +649,8 @@ GDALMDArray::GetGridded(const std::string &osGridOptions,
     }
 
     // Create a in-memory vector layer with (X,Y) points
-    CPLString osTmpFilename;
-    osTmpFilename.Printf("/vsimem/GDALMDArray::GetGridded_%p_%p.%s", this,
-                         pOptions, pszExt);
+    const std::string osTmpFilename(VSIMemGenerateHiddenFilename(
+        std::string("tmp.").append(pszExt).c_str()));
     auto poDS = std::unique_ptr<GDALDataset>(
         poDrv->Create(osTmpFilename.c_str(), 0, 0, 0, GDT_Unknown, nullptr));
     if (!poDS)
@@ -793,7 +776,7 @@ GDALMDArray::GetGridded(const std::string &osGridOptions,
     // CPLDebug("GDAL", "nXSize = %d, nYSize = %d", nXSize, nYSize);
 
     std::vector<std::shared_ptr<GDALDimension>> apoNewDims;
-    const auto apoSelfDims = GetDimensions();
+    const auto &apoSelfDims = GetDimensions();
     for (size_t i = 0; i < GetDimensionCount() - 1; ++i)
         apoNewDims.emplace_back(apoSelfDims[i]);
 
