@@ -12,23 +12,7 @@
  * Copyright (c) 2017, Dmitry Baryshnikov <polimax@mail.ru>
  * Copyright (c) 2017, NextGIS <info@nextgis.com>
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************
  * Object Description Language (ODL) is used to encode data labels for PDS
  * and other NASA data systems. Refer to Chapter 12 of "PDS Standards
@@ -53,7 +37,7 @@
  ****************************************************************************/
 
 #include "nasakeywordhandler.h"
-#include "ogrgeojsonreader.h"
+#include "ogrlibjsonutils.h"
 #include <vector>
 
 //! @cond Doxygen_Suppress
@@ -87,15 +71,16 @@ NASAKeywordHandler::~NASAKeywordHandler()
 /*                               Ingest()                               */
 /************************************************************************/
 
-int NASAKeywordHandler::Ingest(VSILFILE *fp, int nOffset)
+bool NASAKeywordHandler::Ingest(VSILFILE *fp, int nOffset)
 
 {
     /* -------------------------------------------------------------------- */
     /*      Read in buffer till we find END all on its own line.            */
     /* -------------------------------------------------------------------- */
     if (VSIFSeekL(fp, nOffset, SEEK_SET) != 0)
-        return FALSE;
+        return false;
 
+    std::string osHeaderText;
     for (; true;)
     {
         char szChunk[513];
@@ -121,7 +106,17 @@ int NASAKeywordHandler::Ingest(VSILFILE *fp, int nOffset)
             break;
     }
 
-    pszHeaderNext = osHeaderText.c_str();
+    return Parse(osHeaderText.c_str());
+}
+
+/************************************************************************/
+/*                               Parse()                                */
+/************************************************************************/
+
+bool NASAKeywordHandler::Parse(const char *pszStr)
+
+{
+    pszHeaderNext = pszStr;
 
     /* -------------------------------------------------------------------- */
     /*      Process name/value pairs, keeping track of a "path stack".      */
@@ -134,22 +129,22 @@ int NASAKeywordHandler::Ingest(VSILFILE *fp, int nOffset)
 /*                             ReadGroup()                              */
 /************************************************************************/
 
-int NASAKeywordHandler::ReadGroup(const std::string &osPathPrefix,
-                                  CPLJSONObject &oCur, int nRecLevel)
+bool NASAKeywordHandler::ReadGroup(const std::string &osPathPrefix,
+                                   CPLJSONObject &oCur, int nRecLevel)
 
 {
     if (osPathPrefix.size() > 256)
     {
         CPLError(CE_Failure, CPLE_NotSupported, "Too big prefix for GROUP");
-        return FALSE;
+        return false;
     }
     if (nRecLevel == 100)
-        return FALSE;
+        return false;
     for (; true;)
     {
         CPLString osName, osValue;
         if (!ReadPair(osName, osValue, oCur))
-            return FALSE;
+            return false;
 
         if (EQUAL(osName, "OBJECT") || EQUAL(osName, "GROUP"))
         {
@@ -159,7 +154,7 @@ int NASAKeywordHandler::ReadGroup(const std::string &osPathPrefix,
             if (!ReadGroup((osPathPrefix + osValue + ".").c_str(), oNewGroup,
                            nRecLevel + 1))
             {
-                return FALSE;
+                return false;
             }
             CPLJSONObject oName = oNewGroup["Name"];
             if ((osValue == "Table" || osValue == "Field") &&
@@ -186,7 +181,7 @@ int NASAKeywordHandler::ReadGroup(const std::string &osPathPrefix,
         else if (EQUAL(osName, "END") || EQUAL(osName, "END_GROUP") ||
                  EQUAL(osName, "END_OBJECT"))
         {
-            return TRUE;
+            return true;
         }
         else
         {
@@ -216,28 +211,28 @@ static CPLString StripQuotesIfNeeded(const CPLString &osWord,
 /*      Returns TRUE on success.                                        */
 /************************************************************************/
 
-int NASAKeywordHandler::ReadPair(CPLString &osName, CPLString &osValue,
-                                 CPLJSONObject &oCur)
+bool NASAKeywordHandler::ReadPair(CPLString &osName, CPLString &osValue,
+                                  CPLJSONObject &oCur)
 
 {
     osName = "";
     osValue = "";
 
     if (!ReadWord(osName))
-        return FALSE;
+        return false;
 
     SkipWhite();
 
     if (EQUAL(osName, "END"))
-        return TRUE;
+        return true;
 
     if (*pszHeaderNext != '=')
     {
         // ISIS3 does not have anything after the end group/object keyword.
         if (EQUAL(osName, "End_Group") || EQUAL(osName, "End_Object"))
-            return TRUE;
+            return true;
 
-        return FALSE;
+        return false;
     }
 
     pszHeaderNext++;
@@ -302,7 +297,7 @@ int NASAKeywordHandler::ReadPair(CPLString &osName, CPLString &osValue,
                     oStackArrayBeginChar.back() != '(')
                 {
                     CPLDebug("PDS", "Unpaired ( ) for %s", osName.c_str());
-                    return FALSE;
+                    return false;
                 }
                 oStackArrayBeginChar.pop_back();
                 pszHeaderNext++;
@@ -316,7 +311,7 @@ int NASAKeywordHandler::ReadPair(CPLString &osName, CPLString &osValue,
                     oStackArrayBeginChar.back() != '{')
                 {
                     CPLDebug("PDS", "Unpaired { } for %s", osName.c_str());
-                    return FALSE;
+                    return false;
                 }
                 oStackArrayBeginChar.pop_back();
                 pszHeaderNext++;
@@ -344,7 +339,7 @@ int NASAKeywordHandler::ReadPair(CPLString &osName, CPLString &osValue,
     else  // Handle more normal "single word" values.
     {
         if (!ReadWord(osValue, m_bStripSurroundingQuotes, false, &bIsString))
-            return FALSE;
+            return false;
     }
 
     SkipWhite();
@@ -375,7 +370,7 @@ int NASAKeywordHandler::ReadPair(CPLString &osName, CPLString &osValue,
                 }
             }
         }
-        return TRUE;
+        return true;
     }
 
     CPLString osValueNoUnit(osValue);
@@ -425,7 +420,7 @@ int NASAKeywordHandler::ReadPair(CPLString &osName, CPLString &osValue,
     }
     newObject.Add("unit", osUnit);
 
-    return TRUE;
+    return true;
 }
 
 /************************************************************************/
@@ -433,9 +428,9 @@ int NASAKeywordHandler::ReadPair(CPLString &osName, CPLString &osValue,
 /*  Returns TRUE on success                                             */
 /************************************************************************/
 
-int NASAKeywordHandler::ReadWord(CPLString &osWord,
-                                 bool bStripSurroundingQuotes, bool bParseList,
-                                 bool *pbIsString)
+bool NASAKeywordHandler::ReadWord(CPLString &osWord,
+                                  bool bStripSurroundingQuotes, bool bParseList,
+                                  bool *pbIsString)
 
 {
     if (pbIsString)
@@ -446,7 +441,7 @@ int NASAKeywordHandler::ReadWord(CPLString &osWord,
 
     if (!(*pszHeaderNext != '\0' && *pszHeaderNext != '=' &&
           !isspace(static_cast<unsigned char>(*pszHeaderNext))))
-        return FALSE;
+        return false;
 
     /* Extract a text string delimited by '\"' */
     /* Convert newlines (CR or LF) within quotes. While text strings
@@ -461,7 +456,7 @@ int NASAKeywordHandler::ReadWord(CPLString &osWord,
         while (*pszHeaderNext != '"')
         {
             if (*pszHeaderNext == '\0')
-                return FALSE;
+                return false;
             if (*pszHeaderNext == '\n')
             {
                 osWord += "\\n";
@@ -480,7 +475,7 @@ int NASAKeywordHandler::ReadWord(CPLString &osWord,
             osWord += *(pszHeaderNext);
         pszHeaderNext++;
 
-        return TRUE;
+        return true;
     }
 
     /* Extract a symbol string */
@@ -499,14 +494,14 @@ int NASAKeywordHandler::ReadWord(CPLString &osWord,
         while (*pszHeaderNext != '\'')
         {
             if (*pszHeaderNext == '\0')
-                return FALSE;
+                return false;
 
             osWord += *(pszHeaderNext++);
         }
         if (!bStripSurroundingQuotes)
             osWord += *(pszHeaderNext);
         pszHeaderNext++;
-        return TRUE;
+        return true;
     }
 
     /*
@@ -537,7 +532,7 @@ int NASAKeywordHandler::ReadWord(CPLString &osWord,
     if (pbIsString)
         *pbIsString = CPLGetValueType(osWord) == CPL_VALUE_STRING;
 
-    return TRUE;
+    return true;
 }
 
 /************************************************************************/
