@@ -7,23 +7,7 @@
  ******************************************************************************
  * Copyright (c) 2014, Even Rouault <even dot rouault at spatialys dot com>
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #include "cpl_port.h"
@@ -39,16 +23,6 @@
 #include "ogr_p.h"
 #include "ogr_spatialref.h"
 
-/************************************************************************/
-/*                            OGRCurvePolygon()                         */
-/************************************************************************/
-
-/**
- * \brief Create an empty curve polygon.
- */
-
-OGRCurvePolygon::OGRCurvePolygon() = default;
-
 /************************************************************************/
 /*               OGRCurvePolygon( const OGRCurvePolygon& )              */
 /************************************************************************/
@@ -64,12 +38,6 @@ OGRCurvePolygon::OGRCurvePolygon() = default;
 
 OGRCurvePolygon::OGRCurvePolygon(const OGRCurvePolygon &) = default;
 
-/************************************************************************/
-/*                           ~OGRCurvePolygon()                         */
-/************************************************************************/
-
-OGRCurvePolygon::~OGRCurvePolygon() = default;
-
 /************************************************************************/
 /*                 operator=( const OGRCurvePolygon&)                  */
 /************************************************************************/
@@ -89,6 +57,17 @@ OGRCurvePolygon &OGRCurvePolygon::operator=(const OGRCurvePolygon &other)
     {
         OGRSurface::operator=(other);
 
+        for (const auto *poRing : other.oCC)
+        {
+            if (!isRingCorrectType(poRing))
+            {
+                CPLError(CE_Failure, CPLE_AppDefined,
+                         "Illegal use of OGRCurvePolygon::operator=(): "
+                         "trying to assign an incompatible sub-geometry");
+                return *this;
+            }
+        }
+
         oCC = other.oCC;
     }
     return *this;
@@ -342,7 +321,7 @@ OGRErr OGRCurvePolygon::removeRing(int iIndex, bool bDelete)
  * @return OGRERR_NONE in case of success
  */
 
-OGRErr OGRCurvePolygon::addRing(OGRCurve *poNewRing)
+OGRErr OGRCurvePolygon::addRing(const OGRCurve *poNewRing)
 
 {
     OGRCurve *poNewRingCloned = poNewRing->clone();
@@ -352,12 +331,26 @@ OGRErr OGRCurvePolygon::addRing(OGRCurve *poNewRing)
     return eErr;
 }
 
+/************************************************************************/
+/*                            isRingCorrectType()                       */
+/************************************************************************/
+bool OGRCurvePolygon::isRingCorrectType(const OGRCurve *poRing) const
+{
+    return poRing && !EQUAL(poRing->getGeometryName(), "LINEARRING");
+}
+
 /************************************************************************/
 /*                            checkRing()                               */
 /************************************************************************/
 
-int OGRCurvePolygon::checkRing(OGRCurve *poNewRing) const
+bool OGRCurvePolygon::checkRing(const OGRCurve *poNewRing) const
 {
+    if (!isRingCorrectType(poNewRing))
+    {
+        CPLError(CE_Failure, CPLE_AppDefined, "Linearring not allowed.");
+        return false;
+    }
+
     if (!poNewRing->IsEmpty() && !poNewRing->get_IsClosed())
     {
         // This configuration option name must be the same as in
@@ -367,7 +360,7 @@ int OGRCurvePolygon::checkRing(OGRCurve *poNewRing) const
         if (pszEnvVar != nullptr && !CPLTestBool(pszEnvVar))
         {
             CPLError(CE_Failure, CPLE_AppDefined, "Non closed ring detected.");
-            return FALSE;
+            return false;
         }
         else
         {
@@ -384,17 +377,11 @@ int OGRCurvePolygon::checkRing(OGRCurve *poNewRing) const
     {
         if (poNewRing->getNumPoints() < 4)
         {
-            return FALSE;
-        }
-
-        if (EQUAL(poNewRing->getGeometryName(), "LINEARRING"))
-        {
-            CPLError(CE_Failure, CPLE_AppDefined, "Linearring not allowed.");
-            return FALSE;
+            return false;
         }
     }
 
-    return TRUE;
+    return true;
 }
 
 /************************************************************************/
@@ -515,15 +502,16 @@ OGRErr OGRCurvePolygon::importFromWkb(const unsigned char *pabyData,
 /*      Build a well known binary representation of this object.        */
 /************************************************************************/
 
-OGRErr OGRCurvePolygon::exportToWkb(OGRwkbByteOrder eByteOrder,
-                                    unsigned char *pabyData,
-                                    OGRwkbVariant eWkbVariant) const
-
+OGRErr OGRCurvePolygon::exportToWkb(unsigned char *pabyData,
+                                    const OGRwkbExportOptions *psOptions) const
 {
-    if (eWkbVariant == wkbVariantOldOgc)
-        // Does not make sense for new geometries, so patch it.
-        eWkbVariant = wkbVariantIso;
-    return oCC.exportToWkb(this, eByteOrder, pabyData, eWkbVariant);
+    OGRwkbExportOptions sOptions(psOptions ? *psOptions
+                                           : OGRwkbExportOptions());
+
+    // Does not make sense for new geometries, so patch it.
+    if (sOptions.eWkbVariant == wkbVariantOldOgc)
+        sOptions.eWkbVariant = wkbVariantIso;
+    return oCC.exportToWkb(this, pabyData, &sOptions);
 }
 
 /************************************************************************/
@@ -682,6 +670,45 @@ OGRErr OGRCurvePolygon::transform(OGRCoordinateTransformation *poCT)
     return oCC.transform(this, poCT);
 }
 
+/************************************************************************/
+/*                              get_Length()                            */
+/************************************************************************/
+
+double OGRCurvePolygon::get_Length() const
+
+{
+    double dfLength = 0.0;
+    for (const auto &poCurve : *this)
+    {
+        dfLength += poCurve->get_Length();
+    }
+
+    return dfLength;
+}
+
+/************************************************************************/
+/*                        get_GeodesicLength()                          */
+/************************************************************************/
+
+double OGRCurvePolygon::get_GeodesicLength(
+    const OGRSpatialReference *poSRSOverride) const
+
+{
+    if (!poSRSOverride)
+        poSRSOverride = getSpatialReference();
+
+    double dfLength = 0.0;
+    for (const auto &poCurve : *this)
+    {
+        const double dfLocalLength = poCurve->get_GeodesicLength(poSRSOverride);
+        if (dfLocalLength < 0)
+            return dfLocalLength;
+        dfLength += dfLocalLength;
+    }
+
+    return dfLength;
+}
+
 /************************************************************************/
 /*                              get_Area()                              */
 /************************************************************************/
@@ -702,24 +729,51 @@ double OGRCurvePolygon::get_Area() const
     return dfArea;
 }
 
+/************************************************************************/
+/*                        get_GeodesicArea()                            */
+/************************************************************************/
+
+double OGRCurvePolygon::get_GeodesicArea(
+    const OGRSpatialReference *poSRSOverride) const
+
+{
+    if (getExteriorRingCurve() == nullptr)
+        return 0.0;
+
+    if (!poSRSOverride)
+        poSRSOverride = getSpatialReference();
+
+    double dfArea = getExteriorRingCurve()->get_GeodesicArea(poSRSOverride);
+    if (dfArea > 0)
+    {
+        for (int iRing = 0; iRing < getNumInteriorRings(); iRing++)
+        {
+            dfArea -=
+                getInteriorRingCurve(iRing)->get_GeodesicArea(poSRSOverride);
+        }
+    }
+
+    return dfArea;
+}
+
 /************************************************************************/
 /*                       setCoordinateDimension()                       */
 /************************************************************************/
 
-void OGRCurvePolygon::setCoordinateDimension(int nNewDimension)
+bool OGRCurvePolygon::setCoordinateDimension(int nNewDimension)
 
 {
-    oCC.setCoordinateDimension(this, nNewDimension);
+    return oCC.setCoordinateDimension(this, nNewDimension);
 }
 
-void OGRCurvePolygon::set3D(OGRBoolean bIs3D)
+bool OGRCurvePolygon::set3D(OGRBoolean bIs3D)
 {
-    oCC.set3D(this, bIs3D);
+    return oCC.set3D(this, bIs3D);
 }
 
-void OGRCurvePolygon::setMeasured(OGRBoolean bIsMeasured)
+bool OGRCurvePolygon::setMeasured(OGRBoolean bIsMeasured)
 {
-    oCC.setMeasured(this, bIsMeasured);
+    return oCC.setMeasured(this, bIsMeasured);
 }
 
 /************************************************************************/
@@ -744,15 +798,15 @@ OGRBoolean OGRCurvePolygon::IsEmpty() const
 /*                              segmentize()                            */
 /************************************************************************/
 
-void OGRCurvePolygon::segmentize(double dfMaxLength)
+bool OGRCurvePolygon::segmentize(double dfMaxLength)
 {
     if (EQUAL(getGeometryName(), "TRIANGLE"))
     {
         CPLError(CE_Failure, CPLE_NotSupported,
                  "segmentize() is not valid for Triangle");
-        return;
+        return false;
     }
-    oCC.segmentize(dfMaxLength);
+    return oCC.segmentize(dfMaxLength);
 }
 
 /************************************************************************/
@@ -899,4 +953,27 @@ OGRSurfaceCasterToCurvePolygon OGRCurvePolygon::GetCasterToCurvePolygon() const
 {
     return ::CasterToCurvePolygon;
 }
+
 //! @endcond
+
+/************************************************************************/
+/*                           hasEmptyParts()                            */
+/************************************************************************/
+
+bool OGRCurvePolygon::hasEmptyParts() const
+{
+    return oCC.hasEmptyParts();
+}
+
+/************************************************************************/
+/*                          removeEmptyParts()                          */
+/************************************************************************/
+
+void OGRCurvePolygon::removeEmptyParts()
+{
+    auto poExteriorRing = getExteriorRingCurve();
+    if (poExteriorRing && poExteriorRing->IsEmpty())
+        empty();
+    else
+        oCC.removeEmptyParts();
+}
