@@ -7,23 +7,7 @@
  ******************************************************************************
  * Copyright (c) 2022, Planet Labs
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #ifndef OGR_FEATHER_H
@@ -156,6 +140,7 @@ class OGRFeatherWriterLayer final : public OGRArrowWriterLayer
     OGRFeatherWriterLayer(const OGRFeatherWriterLayer &) = delete;
     OGRFeatherWriterLayer &operator=(const OGRFeatherWriterLayer &) = delete;
 
+    GDALDataset *m_poDS = nullptr;
     bool m_bStreamFormat = false;
     std::shared_ptr<arrow::ipc::RecordBatchWriter> m_poFileWriter{};
     std::shared_ptr<arrow::KeyValueMetadata> m_poFooterKeyValueMetadata{};
@@ -164,8 +149,9 @@ class OGRFeatherWriterLayer final : public OGRArrowWriterLayer
     {
         return m_poFileWriter != nullptr;
     }
+
     virtual void CreateWriter() override;
-    virtual void CloseFileWriter() override;
+    virtual bool CloseFileWriter() override;
 
     virtual void CreateSchema() override;
     virtual void PerformStepsBeforeFinalFlushGroup() override;
@@ -179,6 +165,7 @@ class OGRFeatherWriterLayer final : public OGRArrowWriterLayer
 
     virtual bool
     IsSupportedGeometryType(OGRwkbGeometryType eGType) const override;
+
     virtual bool IsSRSRequired() const override
     {
         return true;
@@ -186,7 +173,7 @@ class OGRFeatherWriterLayer final : public OGRArrowWriterLayer
 
   public:
     OGRFeatherWriterLayer(
-        arrow::MemoryPool *poMemoryPool,
+        GDALDataset *poDS, arrow::MemoryPool *poMemoryPool,
         const std::shared_ptr<arrow::io::OutputStream> &poOutputStream,
         const char *pszLayerName);
 
@@ -199,6 +186,11 @@ class OGRFeatherWriterLayer final : public OGRArrowWriterLayer
     bool WriteArrowBatch(const struct ArrowSchema *schema,
                          struct ArrowArray *array,
                          CSLConstList papszOptions = nullptr) override;
+
+    GDALDataset *GetDataset() override
+    {
+        return m_poDS;
+    }
 };
 
 /************************************************************************/
@@ -234,9 +226,8 @@ class OGRFeatherWriterDataset final : public GDALPamDataset
 
   protected:
     OGRLayer *ICreateLayer(const char *pszName,
-                           const OGRSpatialReference *poSpatialRef = nullptr,
-                           OGRwkbGeometryType eGType = wkbUnknown,
-                           char **papszOptions = nullptr) override;
+                           const OGRGeomFieldDefn *poGeomFieldDefn,
+                           CSLConstList papszOptions) override;
 };
 
 #endif  // OGR_FEATHER_H
