@@ -10,23 +10,7 @@
  *  Copyright (c) 2016 Alexandr Borzykh
  *  Copyright (c) 2016 NextGIS, <info@nextgis.com>
  *
- *  Permission is hereby granted, free of charge, to any person obtaining a copy
- *  of this software and associated documentation files (the "Software"), to deal
- *  in the Software without restriction, including without limitation the rights
- *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
- *  copies of the Software, and to permit persons to whom the Software is
- *  furnished to do so, subject to the following conditions:
- *
- *  The above copyright notice and this permission notice shall be included in all
- *  copies or substantial portions of the Software.
- *
- *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
- *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
- *  SOFTWARE.
+  * SPDX-License-Identifier: MIT
  *******************************************************************************/
 
 #include "cpl_port.h"
@@ -232,18 +216,15 @@ long CADHandle::getAsLong() const
 
 long CADHandle::getAsLong(const std::vector<unsigned char>& handle)
 {
-    long result = 0;
+    unsigned long result = 0;
     if( handle.empty() )
         return result;
     size_t copySize = handle.size();
     if( copySize > sizeof(long) )
         copySize = sizeof(long);
-    memcpy( &result, handle.data(), copySize );
-    // NOTE: Second argument below was previously handle.size(). This was
-    // clearly wrong (consider the case where handle.size() > sizeof(long))
-    // but I don't know what the correct value should be
-    SwapEndianness( result, copySize );
-    return result;
+    for( size_t i = 0; i < copySize; ++i )
+        result = result * 256U + handle[i];
+    return static_cast<long>(result);
 }
 
 bool CADHandle::isNull() const
@@ -348,7 +329,15 @@ CADVariant::CADVariant( long julianday, long milliseconds ) :
     dateTimeVal = static_cast<time_t>( dfUnix + dfSeconds );
 
     char str_buff[256] = "Invalid date";
-    struct tm *poLocaltime = localtime(&dateTimeVal);
+#if HAVE_LOCALTIME_R
+    struct tm localtime_tm;
+    const struct tm *poLocaltime = localtime_r(&dateTimeVal, &localtime_tm);
+#elif defined(_WIN32)
+    struct tm localtime_tm;
+    const struct tm *poLocaltime = localtime_s(&localtime_tm, &dateTimeVal) == 0 ? &localtime_tm : nullptr;
+#else
+    const struct tm *poLocaltime = localtime(&dateTimeVal);
+#endif
     if(poLocaltime)
         strftime(str_buff, 255, "%Y-%m-%d %H:%M:%S", poLocaltime);
     stringVal = str_buff;
@@ -504,7 +493,7 @@ const char * CADHeader::getValueName( short code )
 void CADHeader::print() const
 {
     cout << "============ HEADER Section ============\n";
-    for( auto it : valuesMap )
+    for( const auto& it : valuesMap )
     {
         cout << getValueName( it.first ) << ": " << it.second.getString() << "\n";
     }
