@@ -7,23 +7,7 @@
  ******************************************************************************
  * Copyright (c) 2011, Frank Warmerdam <warmerdam@pobox.com>
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #include "ogr_dwg.h"
@@ -58,16 +42,6 @@ OGRDWGDataSource::~OGRDWGDataSource()
     }
 }
 
-/************************************************************************/
-/*                           TestCapability()                           */
-/************************************************************************/
-
-int OGRDWGDataSource::TestCapability(const char * /*pszCap*/)
-
-{
-    return FALSE;
-}
-
 /************************************************************************/
 /*                              GetLayer()                              */
 /************************************************************************/
@@ -93,13 +67,13 @@ int OGRDWGDataSource::Open(OGRDWGServices *poServicesIn,
 
     osEncoding = CPL_ENC_ISO8859_1;
 
-    m_osName = pszFilename;
-
     bInlineBlocks =
         CPLTestBool(CPLGetConfigOption("DWG_INLINE_BLOCKS", "TRUE"));
     bAttributes = CPLTestBool(CPLGetConfigOption("DWG_ATTRIBUTES", "FALSE"));
     bAllAttributes =
         CPLTestBool(CPLGetConfigOption("DWG_ALL_ATTRIBUTES", "TRUE"));
+    m_bClosedLineAsPolygon =
+        CPLTestBool(CPLGetConfigOption("DWG_CLOSED_LINE_AS_POLYGON", "FALSE"));
 
     /* -------------------------------------------------------------------- */
     /*      Open the file.                                                  */
