@@ -8,23 +8,7 @@
  * Copyright (c) 2011, Adam Estrada
  * Copyright (c) 2012, Even Rouault <even dot rouault at spatialys.com>
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #include "ogr_elastic.h"
@@ -32,7 +16,7 @@
 #include "cpl_string.h"
 #include "cpl_csv.h"
 #include "cpl_http.h"
-#include "ogrgeojsonreader.h"
+#include "ogrlibjsonutils.h"
 #include "ogr_swq.h"
 
 /************************************************************************/
@@ -321,7 +305,7 @@ OGRLayer *OGRElasticDataSource::GetLayerByName(const char *pszName)
         return nullptr;
 
     m_apoLayers.push_back(
-        cpl::make_unique<OGRElasticLayer>(pszName, poReferenceLayer));
+        std::make_unique<OGRElasticLayer>(pszName, poReferenceLayer));
     return m_apoLayers.back().get();
 }
 
@@ -412,10 +396,10 @@ OGRErr OGRElasticDataSource::DeleteLayer(int iLayer)
 /*                           ICreateLayer()                             */
 /************************************************************************/
 
-OGRLayer *OGRElasticDataSource::ICreateLayer(const char *pszLayerName,
-                                             OGRSpatialReference *poSRS,
-                                             OGRwkbGeometryType eGType,
-                                             char **papszOptions)
+OGRLayer *
+OGRElasticDataSource::ICreateLayer(const char *pszLayerName,
+                                   const OGRGeomFieldDefn *poGeomFieldDefn,
+                                   CSLConstList papszOptions)
 {
     if (eAccess != GA_Update)
     {
@@ -424,6 +408,10 @@ OGRLayer *OGRElasticDataSource::ICreateLayer(const char *pszLayerName,
         return nullptr;
     }
 
+    const auto eGType = poGeomFieldDefn ? poGeomFieldDefn->GetType() : wkbNone;
+    const auto poSRS =
+        poGeomFieldDefn ? poGeomFieldDefn->GetSpatialRef() : nullptr;
+
     CPLString osLaunderedName(pszLayerName);
 
     const char *pszIndexName = CSLFetchNameValue(papszOptions, "INDEX_NAME");
@@ -675,6 +663,7 @@ CPLHTTPResult *OGRElasticDataSource::HTTPFetch(const char *pszURL,
         }
         aosOptions.SetNameValue("HEADERS", osHeaders.c_str());
     }
+
     return CPLHTTPFetch(pszURL, aosOptions);
 }
 
