@@ -1022,11 +1022,11 @@ static int json_object_double_to_json_string_format(struct json_object *jso, str
 	 * ECMA 262 section 9.8.1 defines
 	 * how to handle these cases as strings
 	 */
-	if (CPLIsNan(jsodbl->c_double))
+	if (isnan(jsodbl->c_double))
 	{
 		size = snprintf(buf, sizeof(buf), "NaN");
 	}
-	else if (CPLIsInf(jsodbl->c_double))
+	else if (isinf(jsodbl->c_double))
 	{
 		if (jsodbl->c_double > 0)
 			size = snprintf(buf, sizeof(buf), "Infinity");
