@@ -8,23 +8,7 @@
  * Copyright (c) 2004, Pirmin Kalberer, Sourcepole AG
  * Copyright (c) 2007-2008, Even Rouault <even dot rouault at spatialys.com>
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #include "cpl_conv.h"
@@ -161,7 +145,7 @@ int OGRILI2DataSource::Open(const char *pszNewName, char **papszOpenOptionsIn,
     }
 
     if (!osModelFilename.empty())
-        poReader->ReadModel(poImdReader, osModelFilename);
+        poReader->ReadModel(this, poImdReader, osModelFilename);
 
     poReader->SetSourceFile(pszName);
 
@@ -266,13 +250,16 @@ int OGRILI2DataSource::Create(const char *pszFilename,
 /*                           ICreateLayer()                             */
 /************************************************************************/
 
-OGRLayer *OGRILI2DataSource::ICreateLayer(
-    const char *pszLayerName, const OGRSpatialReference * /* poSRS */,
-    OGRwkbGeometryType eType, char ** /* papszOptions */)
+OGRLayer *
+OGRILI2DataSource::ICreateLayer(const char *pszLayerName,
+                                const OGRGeomFieldDefn *poGeomFieldDefn,
+                                CSLConstList /*papszOptions*/)
 {
     if (fpOutput == nullptr)
         return nullptr;
 
+    const auto eType = poGeomFieldDefn ? poGeomFieldDefn->GetType() : wkbNone;
+
     FeatureDefnInfo featureDefnInfo =
         poImdReader->GetFeatureDefnInfo(pszLayerName);
     OGRFeatureDefn *poFeatureDefn = featureDefnInfo.GetTableDefnRef();
