@@ -9,23 +9,7 @@
  ******************************************************************************
  * Copyright (c) 1999, Frank Warmerdam
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #include "ntf.h"
@@ -331,8 +315,9 @@ OGRFeature *OGRNTFRasterLayer::GetNextFeature()
 
     iReqColumn =
         static_cast<int>((iCurrentFC - 1) / poReader->GetRasterYSize());
-    iReqRow = static_cast<int>(iCurrentFC -
-                               iReqColumn * poReader->GetRasterYSize() - 1);
+    iReqRow = static_cast<int>(
+        iCurrentFC -
+        static_cast<GIntBig>(iReqColumn) * poReader->GetRasterYSize() - 1);
 
     if (iReqRow + nDEMSample > poReader->GetRasterYSize())
     {
@@ -374,8 +359,9 @@ OGRFeature *OGRNTFRasterLayer::GetFeature(GIntBig nFeatureId)
     /* -------------------------------------------------------------------- */
     iReqColumn =
         static_cast<int>((nFeatureId - 1) / poReader->GetRasterYSize());
-    iReqRow = static_cast<int>(nFeatureId -
-                               iReqColumn * poReader->GetRasterYSize() - 1);
+    iReqRow = static_cast<int>(
+        nFeatureId -
+        static_cast<GIntBig>(iReqColumn) * poReader->GetRasterYSize() - 1);
 
     if (iReqColumn != iColumnOffset)
     {
