@@ -7,41 +7,22 @@
  ******************************************************************************
  * Copyright (c) 2010-2011, Even Rouault <even dot rouault at spatialys.com>
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #include "cpl_conv.h"
 #include "cpl_string.h"
 #include "ogr_pds.h"
 
-
 using namespace OGRPDS;
 
 /************************************************************************/
 /*                           OGRPDSDataSource()                         */
 /************************************************************************/
 
-OGRPDSDataSource::OGRPDSDataSource() :
-    pszName(nullptr),
-    papoLayers(nullptr),
-    nLayers(0)
-{}
+OGRPDSDataSource::OGRPDSDataSource() : papoLayers(nullptr), nLayers(0)
+{
+}
 
 /************************************************************************/
 /*                        ~OGRPDSDataSource()                           */
@@ -50,30 +31,19 @@ OGRPDSDataSource::OGRPDSDataSource() :
 OGRPDSDataSource::~OGRPDSDataSource()
 
 {
-    for( int i = 0; i < nLayers; i++ )
+    for (int i = 0; i < nLayers; i++)
         delete papoLayers[i];
-    CPLFree( papoLayers );
-
-    CPLFree( pszName );
-}
-
-/************************************************************************/
-/*                           TestCapability()                           */
-/************************************************************************/
-
-int OGRPDSDataSource::TestCapability( const char * /* pszCap */ )
-{
-    return FALSE;
+    CPLFree(papoLayers);
 }
 
 /************************************************************************/
 /*                              GetLayer()                              */
 /************************************************************************/
 
-OGRLayer *OGRPDSDataSource::GetLayer( int iLayer )
+OGRLayer *OGRPDSDataSource::GetLayer(int iLayer)
 
 {
-    if( iLayer < 0 || iLayer >= nLayers )
+    if (iLayer < 0 || iLayer >= nLayers)
         return nullptr;
 
     return papoLayers[iLayer];
@@ -83,30 +53,29 @@ OGRLayer *OGRPDSDataSource::GetLayer( int iLayer )
 /*                          GetKeywordSub()                             */
 /************************************************************************/
 
-const char * OGRPDSDataSource::GetKeywordSub( const char *pszPath,
-                                              int iSubscript,
-                                              const char *pszDefault )
+const char *OGRPDSDataSource::GetKeywordSub(const char *pszPath, int iSubscript,
+                                            const char *pszDefault)
 
 {
-    const char *pszResult = oKeywords.GetKeyword( pszPath, nullptr );
+    const char *pszResult = oKeywords.GetKeyword(pszPath, nullptr);
 
-    if( pszResult == nullptr )
+    if (pszResult == nullptr)
         return pszDefault;
 
-    if( pszResult[0] != '(' )
+    if (pszResult[0] != '(')
         return pszDefault;
 
-    char **papszTokens = CSLTokenizeString2( pszResult, "(,)",
-                                             CSLT_HONOURSTRINGS );
+    char **papszTokens =
+        CSLTokenizeString2(pszResult, "(,)", CSLT_HONOURSTRINGS);
 
-    if( iSubscript <= CSLCount(papszTokens) )
+    if (iSubscript <= CSLCount(papszTokens))
     {
-        osTempResult = papszTokens[iSubscript-1];
-        CSLDestroy( papszTokens );
+        osTempResult = papszTokens[iSubscript - 1];
+        CSLDestroy(papszTokens);
         return osTempResult.c_str();
     }
 
-    CSLDestroy( papszTokens );
+    CSLDestroy(papszTokens);
     return pszDefault;
 }
 
@@ -117,26 +86,26 @@ const char * OGRPDSDataSource::GetKeywordSub( const char *pszPath,
 /* The change is made in-place to CPLString.                            */
 /************************************************************************/
 
-void OGRPDSDataSource::CleanString( CPLString &osInput )
+void OGRPDSDataSource::CleanString(CPLString &osInput)
 
 {
-    if( ( osInput.size() < 2 ) ||
-      ((osInput.at(0) != '"'   || osInput.at(osInput.size()-1) != '"' ) &&
-       ( osInput.at(0) != '\'' || osInput.at(osInput.size()-1) != '\'')) )
-         return;
+    if ((osInput.size() < 2) ||
+        ((osInput.at(0) != '"' || osInput.at(osInput.size() - 1) != '"') &&
+         (osInput.at(0) != '\'' || osInput.at(osInput.size() - 1) != '\'')))
+        return;
 
     char *pszWrk = CPLStrdup(osInput.c_str() + 1);
 
-    pszWrk[strlen(pszWrk)-1] = '\0';
+    pszWrk[strlen(pszWrk) - 1] = '\0';
 
-    for( int i = 0; pszWrk[i] != '\0'; i++ )
+    for (int i = 0; pszWrk[i] != '\0'; i++)
     {
-        if( pszWrk[i] == ' ' )
+        if (pszWrk[i] == ' ')
             pszWrk[i] = '_';
     }
 
     osInput = pszWrk;
-    CPLFree( pszWrk );
+    CPLFree(pszWrk);
 }
 
 /************************************************************************/
@@ -148,9 +117,8 @@ static CPLString MakeAttr(CPLString os1, CPLString os2)
     return os1 + "." + os2;
 }
 
-bool OGRPDSDataSource::LoadTable( const char* pszFilename,
-                                  int nRecordSize,
-                                  CPLString osTableID )
+bool OGRPDSDataSource::LoadTable(const char *pszFilename, int nRecordSize,
+                                 CPLString osTableID)
 {
     CPLString osTableFilename;
     int nStartBytes = 0;
@@ -158,49 +126,47 @@ bool OGRPDSDataSource::LoadTable( const char* pszFilename,
     CPLString osTableLink = "^";
     osTableLink += osTableID;
 
-    CPLString osTable = oKeywords.GetKeyword( osTableLink, "" );
-    if( osTable[0] == '(' )
+    CPLString osTable = oKeywords.GetKeyword(osTableLink, "");
+    if (osTable[0] == '(')
     {
         osTableFilename = GetKeywordSub(osTableLink, 1, "");
         CPLString osStartRecord = GetKeywordSub(osTableLink, 2, "");
         nStartBytes = atoi(osStartRecord.c_str());
-        if( nStartBytes <= 0 ||
-            (( nRecordSize > 0 && nStartBytes > INT_MAX / nRecordSize )) )
+        if (nStartBytes <= 0 ||
+            ((nRecordSize > 0 && nStartBytes > INT_MAX / nRecordSize)))
         {
-            CPLError(CE_Failure, CPLE_NotSupported,
-                     "Invalid StartBytes value");
+            CPLError(CE_Failure, CPLE_NotSupported, "Invalid StartBytes value");
             return false;
         }
-        nStartBytes --;
+        nStartBytes--;
         nStartBytes *= nRecordSize;
-        if (osTableFilename.empty() || osStartRecord.empty() ||
-            nStartBytes < 0)
+        if (osTableFilename.empty() || osStartRecord.empty() || nStartBytes < 0)
         {
-            CPLError(CE_Failure, CPLE_NotSupported,
-                    "Cannot parse %s line", osTableLink.c_str());
+            CPLError(CE_Failure, CPLE_NotSupported, "Cannot parse %s line",
+                     osTableLink.c_str());
             return false;
         }
         CPLString osTPath = CPLGetPath(pszFilename);
-        CleanString( osTableFilename );
-        osTableFilename = CPLFormCIFilename( osTPath, osTableFilename, nullptr );
+        CleanString(osTableFilename);
+        osTableFilename = CPLFormCIFilename(osTPath, osTableFilename, nullptr);
     }
     else
     {
-        osTableFilename = oKeywords.GetKeyword( osTableLink, "" );
+        osTableFilename = oKeywords.GetKeyword(osTableLink, "");
         if (!osTableFilename.empty() && osTableFilename[0] >= '0' &&
             osTableFilename[0] <= '9')
         {
             nStartBytes = atoi(osTableFilename.c_str());
-            if( nStartBytes <= 1 )
+            if (nStartBytes <= 1)
             {
-                CPLError(CE_Failure, CPLE_NotSupported,
-                        "Cannot parse %s line", osTableFilename.c_str());
+                CPLError(CE_Failure, CPLE_NotSupported, "Cannot parse %s line",
+                         osTableFilename.c_str());
                 return false;
             }
             nStartBytes = nStartBytes - 1;
             if (strstr(osTableFilename.c_str(), "<BYTES>") == nullptr)
             {
-                if( nRecordSize > 0 && nStartBytes > INT_MAX / nRecordSize )
+                if (nRecordSize > 0 && nStartBytes > INT_MAX / nRecordSize)
                 {
                     CPLError(CE_Failure, CPLE_NotSupported,
                              "Too big StartBytes value");
@@ -213,31 +179,29 @@ bool OGRPDSDataSource::LoadTable( const char* pszFilename,
         else
         {
             CPLString osTPath = CPLGetPath(pszFilename);
-            CleanString( osTableFilename );
+            CleanString(osTableFilename);
             osTableFilename =
-                CPLFormCIFilename( osTPath, osTableFilename, nullptr );
+                CPLFormCIFilename(osTPath, osTableFilename, nullptr);
             nStartBytes = 0;
         }
     }
 
     CPLString osTableName =
-        oKeywords.GetKeyword( MakeAttr(osTableID, "NAME"), "" );
+        oKeywords.GetKeyword(MakeAttr(osTableID, "NAME"), "");
     if (osTableName.empty())
     {
         if (GetLayerByName(osTableID.c_str()) == nullptr)
             osTableName = osTableID;
         else
-            osTableName = CPLSPrintf("Layer_%d", nLayers+1);
+            osTableName = CPLSPrintf("Layer_%d", nLayers + 1);
     }
     CleanString(osTableName);
     CPLString osTableInterchangeFormat =
-            oKeywords.GetKeyword( MakeAttr(osTableID, "INTERCHANGE_FORMAT"),
-                                  "" );
+        oKeywords.GetKeyword(MakeAttr(osTableID, "INTERCHANGE_FORMAT"), "");
     CPLString osTableRows =
-        oKeywords.GetKeyword( MakeAttr(osTableID, "ROWS"), "" );
+        oKeywords.GetKeyword(MakeAttr(osTableID, "ROWS"), "");
     const int nRecords = atoi(osTableRows);
-    if (osTableInterchangeFormat.empty() ||
-        osTableRows.empty() || nRecords < 0)
+    if (osTableInterchangeFormat.empty() || osTableRows.empty() || nRecords < 0)
     {
         CPLError(CE_Failure, CPLE_NotSupported,
                  "One of TABLE.INTERCHANGE_FORMAT or TABLE.ROWS is missing");
@@ -253,7 +217,7 @@ bool OGRPDSDataSource::LoadTable( const char* pszFilename,
         return false;
     }
 
-    VSILFILE* fp = VSIFOpenL(osTableFilename, "rb");
+    VSILFILE *fp = VSIFOpenL(osTableFilename, "rb");
     if (fp == nullptr)
     {
         CPLError(CE_Failure, CPLE_AppDefined, "Cannot open %s",
@@ -262,15 +226,16 @@ bool OGRPDSDataSource::LoadTable( const char* pszFilename,
     }
 
     CPLString osTableStructure =
-        oKeywords.GetKeyword( MakeAttr(osTableID, "^STRUCTURE"), "" );
+        oKeywords.GetKeyword(MakeAttr(osTableID, "^STRUCTURE"), "");
     if (!osTableStructure.empty())
     {
         CPLString osTPath = CPLGetPath(pszFilename);
-        CleanString( osTableStructure );
-        osTableStructure = CPLFormCIFilename( osTPath, osTableStructure, nullptr );
+        CleanString(osTableStructure);
+        osTableStructure =
+            CPLFormCIFilename(osTPath, osTableStructure, nullptr);
     }
 
-    GByte* pabyRecord = (GByte*) VSI_MALLOC_VERBOSE(nRecordSize + 1);
+    GByte *pabyRecord = (GByte *)VSI_MALLOC_VERBOSE(nRecordSize + 1);
     if (pabyRecord == nullptr)
     {
         VSIFCloseL(fp);
@@ -278,14 +243,11 @@ bool OGRPDSDataSource::LoadTable( const char* pszFilename,
     }
     pabyRecord[nRecordSize] = 0;
 
-    papoLayers = static_cast<OGRLayer**>(
-        CPLRealloc(papoLayers, (nLayers + 1) * sizeof(OGRLayer*)));
+    papoLayers = static_cast<OGRLayer **>(
+        CPLRealloc(papoLayers, (nLayers + 1) * sizeof(OGRLayer *)));
     papoLayers[nLayers] = new OGRPDSLayer(
-        osTableID, osTableName, fp,
-        pszFilename,
-        osTableStructure,
-        nRecords, nStartBytes,
-        nRecordSize, pabyRecord,
+        osTableID, osTableName, fp, pszFilename, osTableStructure, nRecords,
+        nStartBytes, nRecordSize, pabyRecord,
         osTableInterchangeFormat.compare("ASCII") == 0);
     nLayers++;
 
@@ -296,25 +258,23 @@ bool OGRPDSDataSource::LoadTable( const char* pszFilename,
 /*                                Open()                                */
 /************************************************************************/
 
-int OGRPDSDataSource::Open( const char * pszFilename )
+int OGRPDSDataSource::Open(const char *pszFilename)
 
 {
-    pszName = CPLStrdup( pszFilename );
+    // --------------------------------------------------------------------
+    //      Does this appear to be a .PDS table file?
+    // --------------------------------------------------------------------
 
-// --------------------------------------------------------------------
-//      Does this appear to be a .PDS table file?
-// --------------------------------------------------------------------
-
-    VSILFILE* fp = VSIFOpenL(pszFilename, "rb");
+    VSILFILE *fp = VSIFOpenL(pszFilename, "rb");
     if (fp == nullptr)
         return FALSE;
 
     char szBuffer[512];
-    int nbRead = static_cast<int>(
-        VSIFReadL(szBuffer, 1, sizeof(szBuffer) - 1, fp));
+    int nbRead =
+        static_cast<int>(VSIFReadL(szBuffer, 1, sizeof(szBuffer) - 1, fp));
     szBuffer[nbRead] = '\0';
 
-    const char* pszPos = strstr(szBuffer, "PDS_VERSION_ID");
+    const char *pszPos = strstr(szBuffer, "PDS_VERSION_ID");
     const bool bIsPDS = pszPos != nullptr;
 
     if (!bIsPDS)
@@ -330,13 +290,13 @@ int OGRPDSDataSource::Open( const char * pszFilename )
     }
 
     VSIFCloseL(fp);
-    CPLString osRecordType = oKeywords.GetKeyword( "RECORD_TYPE", "" );
-    CPLString osFileRecords = oKeywords.GetKeyword( "FILE_RECORDS", "" );
-    CPLString osRecordBytes = oKeywords.GetKeyword( "RECORD_BYTES", "" );
+    CPLString osRecordType = oKeywords.GetKeyword("RECORD_TYPE", "");
+    CPLString osFileRecords = oKeywords.GetKeyword("FILE_RECORDS", "");
+    CPLString osRecordBytes = oKeywords.GetKeyword("RECORD_BYTES", "");
     int nRecordSize = atoi(osRecordBytes);
     if (osRecordType.empty() || osFileRecords.empty() ||
         osRecordBytes.empty() || nRecordSize <= 0 ||
-        nRecordSize > 10*1024*1024)
+        nRecordSize > 10 * 1024 * 1024)
     {
         CPLError(CE_Failure, CPLE_NotSupported,
                  "One of RECORD_TYPE, FILE_RECORDS or RECORD_BYTES is missing");
@@ -350,7 +310,7 @@ int OGRPDSDataSource::Open( const char * pszFilename )
         return FALSE;
     }
 
-    CPLString osTable = oKeywords.GetKeyword( "^TABLE", "" );
+    CPLString osTable = oKeywords.GetKeyword("^TABLE", "");
     if (!osTable.empty())
     {
         LoadTable(pszFilename, nRecordSize, "TABLE");
@@ -363,24 +323,23 @@ int OGRPDSDataSource::Open( const char * pszFilename )
 
         // To avoid performance issues with datasets generated by oss-fuzz
         int nErrors = 0;
-        while( nErrors < 10 )
+        while (nErrors < 10)
         {
             CPLPushErrorHandler(CPLQuietErrorHandler);
-            const char* pszLine = CPLReadLine2L(fp, 256, nullptr);
+            const char *pszLine = CPLReadLine2L(fp, 256, nullptr);
             CPLPopErrorHandler();
             CPLErrorReset();
             if (pszLine == nullptr)
                 break;
-            char** papszTokens =
-                CSLTokenizeString2( pszLine, " =", CSLT_HONOURSTRINGS );
+            char **papszTokens =
+                CSLTokenizeString2(pszLine, " =", CSLT_HONOURSTRINGS);
             int nTokens = CSLCount(papszTokens);
-            if (nTokens == 2 &&
-                papszTokens[0][0] == '^' &&
+            if (nTokens == 2 && papszTokens[0][0] == '^' &&
                 strstr(papszTokens[0], "TABLE") != nullptr)
             {
-                if( !LoadTable(pszFilename, nRecordSize, papszTokens[0] + 1) )
+                if (!LoadTable(pszFilename, nRecordSize, papszTokens[0] + 1))
                 {
-                    nErrors ++;
+                    nErrors++;
                 }
             }
             CSLDestroy(papszTokens);
