@@ -8,23 +8,7 @@
  * Copyright (c) 1999,  Les Technologies SoftMap Inc.
  * Copyright (c) 2008-2013, Even Rouault <even dot rouault at spatialys.com>
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #include "cpl_port.h"
@@ -97,8 +81,8 @@ DBFHandle OGRShapeDataSource::DS_DBFOpen(const char *pszDBFFile,
 /************************************************************************/
 
 OGRShapeDataSource::OGRShapeDataSource()
-    : papoLayers(nullptr), nLayers(0), pszName(nullptr),
-      bSingleFileDataSource(false), poPool(new OGRLayerPool()),
+    : papoLayers(nullptr), nLayers(0), bSingleFileDataSource(false),
+      poPool(new OGRLayerPool()),
       b2GBLimit(CPLTestBool(CPLGetConfigOption("SHAPE_2GB_LIMIT", "FALSE")))
 {
 }
@@ -156,8 +140,6 @@ OGRShapeDataSource::~OGRShapeDataSource()
         CPLDestroyCond(m_poRefreshLockFileCond);
         m_poRefreshLockFileCond = nullptr;
     }
-
-    CPLFree(pszName);
 }
 
 /************************************************************************/
@@ -169,14 +151,15 @@ bool OGRShapeDataSource::OpenZip(GDALOpenInfo *poOpenInfo,
 {
     if (!Open(poOpenInfo, true))
         return false;
-    CPLFree(pszName);
-    pszName = CPLStrdup(pszOriFilename);
+
+    SetDescription(pszOriFilename);
+
     m_bIsZip = true;
     m_bSingleLayerZip = EQUAL(CPLGetExtension(pszOriFilename), "shz");
 
     if (!m_bSingleLayerZip)
     {
-        CPLString osLockFile(pszName);
+        CPLString osLockFile(GetDescription());
         osLockFile += ".gdal.lock";
         VSIStatBufL sStat;
         if (VSIStatL(osLockFile, &sStat) == 0 &&
@@ -197,9 +180,8 @@ bool OGRShapeDataSource::OpenZip(GDALOpenInfo *poOpenInfo,
 bool OGRShapeDataSource::CreateZip(const char *pszOriFilename)
 {
     CPLAssert(nLayers == 0);
-    pszName = CPLStrdup(pszOriFilename);
 
-    void *hZIP = CPLCreateZip(pszName, nullptr);
+    void *hZIP = CPLCreateZip(pszOriFilename, nullptr);
     if (!hZIP)
         return false;
     if (CPLCloseZip(hZIP) != CE_None)
@@ -225,8 +207,6 @@ bool OGRShapeDataSource::Open(GDALOpenInfo *poOpenInfo, bool bTestOpen,
     CPLAssert(papszOpenOptions == nullptr);
     papszOpenOptions = CSLDuplicate(poOpenInfo->papszOpenOptions);
 
-    pszName = CPLStrdup(pszNewName);
-
     eAccess = poOpenInfo->eAccess;
 
     bSingleFileDataSource = CPL_TO_BOOL(bForceSingleFileDataSource);
@@ -288,7 +268,7 @@ bool OGRShapeDataSource::Open(GDALOpenInfo *poOpenInfo, bool bTestOpen,
             const char *pszCandidate = papszCandidates[iCan];
             const char *pszLayerName = CPLGetBasename(pszCandidate);
             CPLString osLayerName(pszLayerName);
-#ifdef WIN32
+#ifdef _WIN32
             // On Windows, as filenames are case insensitive, a shapefile layer
             // can be made of foo.shp and FOO.DBF, so to detect unique layer
             // names, put them upper case in the unique set used for detection.
@@ -330,7 +310,7 @@ bool OGRShapeDataSource::Open(GDALOpenInfo *poOpenInfo, bool bTestOpen,
             const char *pszCandidate = papszCandidates[iCan];
             const char *pszLayerName = CPLGetBasename(pszCandidate);
             CPLString osLayerName(pszLayerName);
-#ifdef WIN32
+#ifdef _WIN32
             osLayerName.toupper();
 #endif
 
@@ -570,15 +550,19 @@ static CPLString LaunderLayerName(const char *pszLayerName)
 /*                           ICreateLayer()                             */
 /************************************************************************/
 
-OGRLayer *OGRShapeDataSource::ICreateLayer(const char *pszLayerName,
-                                           const OGRSpatialReference *poSRS,
-                                           OGRwkbGeometryType eType,
-                                           char **papszOptions)
+OGRLayer *
+OGRShapeDataSource::ICreateLayer(const char *pszLayerName,
+                                 const OGRGeomFieldDefn *poGeomFieldDefn,
+                                 CSLConstList papszOptions)
 
 {
     // To ensure that existing layers are created.
     GetLayerCount();
 
+    auto eType = poGeomFieldDefn ? poGeomFieldDefn->GetType() : wkbNone;
+    const auto poSRS =
+        poGeomFieldDefn ? poGeomFieldDefn->GetSpatialRef() : nullptr;
+
     /* -------------------------------------------------------------------- */
     /*      Check that the layer doesn't already exist.                     */
     /* -------------------------------------------------------------------- */
@@ -597,7 +581,7 @@ OGRLayer *OGRShapeDataSource::ICreateLayer(const char *pszLayerName,
         CPLError(CE_Failure, CPLE_NoWriteAccess,
                  "Data source %s opened read-only.  "
                  "New layer %s cannot be created.",
-                 pszName, pszLayerName);
+                 GetDescription(), pszLayerName);
 
         return nullptr;
     }
@@ -799,8 +783,8 @@ OGRLayer *OGRShapeDataSource::ICreateLayer(const char *pszLayerName,
 
     if (bSingleFileDataSource && nLayers == 0)
     {
-        char *pszPath = CPLStrdup(CPLGetPath(pszName));
-        char *pszFBasename = CPLStrdup(CPLGetBasename(pszName));
+        char *pszPath = CPLStrdup(CPLGetPath(GetDescription()));
+        char *pszFBasename = CPLStrdup(CPLGetBasename(GetDescription()));
 
         pszFilenameWithoutExt =
             CPLStrdup(CPLFormFilename(pszPath, pszFBasename, nullptr));
@@ -816,17 +800,18 @@ OGRLayer *OGRShapeDataSource::ICreateLayer(const char *pszLayerName,
         // directory as 'foo.shp'
         // So technically, we will not be any longer a single file
         // datasource ... Ahem ahem.
-        char *pszPath = CPLStrdup(CPLGetPath(pszName));
+        char *pszPath = CPLStrdup(CPLGetPath(GetDescription()));
         pszFilenameWithoutExt = CPLStrdup(CPLFormFilename(
             pszPath, LaunderLayerName(pszLayerName).c_str(), nullptr));
         CPLFree(pszPath);
     }
     else
     {
-        CPLString osDir(m_osTemporaryUnzipDir.empty() ? pszName
-                                                      : m_osTemporaryUnzipDir);
+        const std::string osDir(m_osTemporaryUnzipDir.empty()
+                                    ? std::string(GetDescription())
+                                    : m_osTemporaryUnzipDir);
         pszFilenameWithoutExt = CPLStrdup(CPLFormFilename(
-            osDir, LaunderLayerName(pszLayerName).c_str(), nullptr));
+            osDir.c_str(), LaunderLayerName(pszLayerName).c_str(), nullptr));
     }
 
     /* -------------------------------------------------------------------- */
@@ -1076,7 +1061,7 @@ OGRLayer *OGRShapeDataSource::GetLayerByName(const char *pszLayerNameIn)
     }
 #endif
 
-    return OGRDataSource::GetLayerByName(pszLayerNameIn);
+    return GDALDataset::GetLayerByName(pszLayerNameIn);
 }
 
 /************************************************************************/
@@ -1211,8 +1196,8 @@ OGRLayer *OGRShapeDataSource::ExecuteSQL(const char *pszStatement,
         }
         CSLDestroy(papszTokens);
 
-        return OGRDataSource::ExecuteSQL(pszStatement, poSpatialFilter,
-                                         pszDialect);
+        return GDALDataset::ExecuteSQL(pszStatement, poSpatialFilter,
+                                       pszDialect);
     }
 
     /* -------------------------------------------------------------------- */
@@ -1288,7 +1273,7 @@ OGRErr OGRShapeDataSource::DeleteLayer(int iLayer)
         CPLError(CE_Failure, CPLE_NoWriteAccess,
                  "Data source %s opened read-only.  "
                  "Layer %d cannot be deleted.",
-                 pszName, iLayer);
+                 GetDescription(), iLayer);
 
         return OGRERR_FAILURE;
     }
@@ -1378,7 +1363,7 @@ char **OGRShapeDataSource::GetFileList()
 {
     if (m_bIsZip)
     {
-        return CSLAddString(nullptr, pszName);
+        return CSLAddString(nullptr, GetDescription());
     }
     CPLStringList oFileList;
     GetLayerCount();
@@ -1399,6 +1384,7 @@ void OGRShapeDataSource::RefreshLockFile(void *_self)
     OGRShapeDataSource *self = static_cast<OGRShapeDataSource *>(_self);
     CPLAssert(self->m_psLockFile);
     CPLAcquireMutex(self->m_poRefreshLockFileMutex, 1000);
+    self->m_bRefreshLockFileThreadStarted = true;
     CPLCondSignal(self->m_poRefreshLockFileCond);
     unsigned int nInc = 0;
     while (!(self->m_bExitRefreshLockFileThread))
@@ -1441,7 +1427,7 @@ void OGRShapeDataSource::RemoveLockFile()
     // Close and remove lock file
     VSIFCloseL(m_psLockFile);
     m_psLockFile = nullptr;
-    CPLString osLockFile(pszName);
+    CPLString osLockFile(GetDescription());
     osLockFile += ".gdal.lock";
     VSIUnlink(osLockFile);
 }
@@ -1459,20 +1445,22 @@ bool OGRShapeDataSource::UncompressIfNeeded()
 
     auto returnError = [this]()
     {
-        CPLError(CE_Failure, CPLE_AppDefined, "Cannot uncompress %s", pszName);
+        CPLError(CE_Failure, CPLE_AppDefined, "Cannot uncompress %s",
+                 GetDescription());
         return false;
     };
 
     if (nLayers > 1)
     {
-        CPLString osLockFile(pszName);
+        CPLString osLockFile(GetDescription());
         osLockFile += ".gdal.lock";
         VSIStatBufL sStat;
         if (VSIStatL(osLockFile, &sStat) == 0 &&
             sStat.st_mtime > time(nullptr) - 2 * knREFRESH_LOCK_FILE_DELAY_SEC)
         {
             CPLError(CE_Failure, CPLE_AppDefined,
-                     "Cannot edit %s. Another task is editing it", pszName);
+                     "Cannot edit %s. Another task is editing it",
+                     GetDescription());
             return false;
         }
         if (!m_poRefreshLockFileMutex)
@@ -1495,7 +1483,10 @@ bool OGRShapeDataSource::UncompressIfNeeded()
             return false;
         }
         m_psLockFile = f;
+        CPLAcquireMutex(m_poRefreshLockFileMutex, 1000);
         m_bExitRefreshLockFileThread = false;
+        m_bRefreshLockFileThreadStarted = false;
+        CPLReleaseMutex(m_poRefreshLockFileMutex);
         // Config option mostly for testing purposes
         // coverity[tainted_data]
         m_dfRefreshLockDelay = CPLAtof(CPLGetConfigOption(
@@ -1512,7 +1503,10 @@ bool OGRShapeDataSource::UncompressIfNeeded()
         else
         {
             CPLAcquireMutex(m_poRefreshLockFileMutex, 1000);
-            CPLCondWait(m_poRefreshLockFileCond, m_poRefreshLockFileMutex);
+            while (!m_bRefreshLockFileThreadStarted)
+            {
+                CPLCondWait(m_poRefreshLockFileCond, m_poRefreshLockFileMutex);
+            }
             CPLReleaseMutex(m_poRefreshLockFileMutex);
         }
     }
@@ -1535,7 +1529,7 @@ bool OGRShapeDataSource::UncompressIfNeeded()
         }
     }
 
-    CPLString osTemporaryDir(pszName);
+    CPLString osTemporaryDir(GetDescription());
     osTemporaryDir += "_tmp_uncompressed";
 
     const char *pszUseVsimem =
@@ -1545,7 +1539,7 @@ bool OGRShapeDataSource::UncompressIfNeeded()
          nTotalUncompressedSize <
              static_cast<GUIntBig>(CPLGetUsablePhysicalRAM() / 10)))
     {
-        osTemporaryDir = CPLSPrintf("/vsimem/_shapedriver/%p", this);
+        osTemporaryDir = VSIMemGenerateHiddenFilename("shapedriver");
     }
     CPLDebug("Shape", "Uncompressing to %s", osTemporaryDir.c_str());
 
@@ -1569,7 +1563,7 @@ bool OGRShapeDataSource::UncompressIfNeeded()
         }
     }
 
-    m_osTemporaryUnzipDir = osTemporaryDir;
+    m_osTemporaryUnzipDir = std::move(osTemporaryDir);
 
     for (int i = 0; i < nLayers; i++)
     {
@@ -1592,7 +1586,8 @@ bool OGRShapeDataSource::RecompressIfNeeded(
 
     auto returnError = [this]()
     {
-        CPLError(CE_Failure, CPLE_AppDefined, "Cannot recompress %s", pszName);
+        CPLError(CE_Failure, CPLE_AppDefined, "Cannot recompress %s",
+                 GetDescription());
         RemoveLockFile();
         return false;
     };
@@ -1683,7 +1678,7 @@ bool OGRShapeDataSource::RecompressIfNeeded(
 
     const bool bOverwrite =
         CPLTestBool(CPLGetConfigOption("OGR_SHAPE_PACK_IN_PLACE",
-#ifdef WIN32
+#ifdef _WIN32
                                        "YES"
 #else
                                        "NO"
@@ -1694,7 +1689,7 @@ bool OGRShapeDataSource::RecompressIfNeeded(
         VSILFILE *fpTarget = nullptr;
         for (int i = 0; i < 10; i++)
         {
-            fpTarget = VSIFOpenL(pszName, "rb+");
+            fpTarget = VSIFOpenL(GetDescription(), "rb+");
             if (fpTarget)
                 break;
             CPLSleep(0.1);
@@ -1711,7 +1706,8 @@ bool OGRShapeDataSource::RecompressIfNeeded(
     }
     else
     {
-        if (VSIUnlink(pszName) != 0 || CPLMoveFile(pszName, osTmpZip) != 0)
+        if (VSIUnlink(GetDescription()) != 0 ||
+            CPLMoveFile(GetDescription(), osTmpZip) != 0)
         {
             return returnError();
         }
