@@ -9,41 +9,13 @@
  * Copyright (c) 2016, Avyav Kumar Singh <avyavkumar at gmail dot com>
  * Copyright (c) 2016, Even Rouault <even.roauult at spatialys.com>
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #include "ogr_geometry.h"
 #include "ogr_p.h"
 #include "ogr_api.h"
 
-
-/************************************************************************/
-/*                        OGRTriangulatedSurface()                      */
-/************************************************************************/
-
-/**
- * \brief Constructor.
- *
- */
-
-OGRTriangulatedSurface::OGRTriangulatedSurface() = default;
-
 /************************************************************************/
 /*        OGRTriangulatedSurface( const OGRTriangulatedSurface& )       */
 /************************************************************************/
@@ -54,23 +26,12 @@ OGRTriangulatedSurface::OGRTriangulatedSurface() = default;
  */
 
 OGRTriangulatedSurface::OGRTriangulatedSurface(
-                                        const OGRTriangulatedSurface& other ) :
-    OGRPolyhedralSurface()
+    const OGRTriangulatedSurface &other)
+    : OGRPolyhedralSurface()
 {
     *this = other;
 }
 
-/************************************************************************/
-/*                        ~OGRTriangulatedSurface()                     */
-/************************************************************************/
-
-/**
- * \brief Destructor
- *
- */
-
-OGRTriangulatedSurface::~OGRTriangulatedSurface() = default;
-
 /************************************************************************/
 /*                 operator=( const OGRTriangulatedSurface&)            */
 /************************************************************************/
@@ -80,22 +41,18 @@ OGRTriangulatedSurface::~OGRTriangulatedSurface() = default;
  *
  */
 
-OGRTriangulatedSurface& OGRTriangulatedSurface::operator=(
-                                        const OGRTriangulatedSurface& other )
+OGRTriangulatedSurface &
+OGRTriangulatedSurface::operator=(const OGRTriangulatedSurface &other)
 {
-    if( this != &other)
+    if (this != &other)
     {
         // We need to do it manually. We cannot rely on the = operator
         // of OGRPolyhedralSurface since it will be confused by a multipolygon
         // of triangles.
-        OGRSurface::operator=( other );
-        empty();
-        set3D( other.Is3D() );
-        setMeasured( other.IsMeasured() );
-        assignSpatialReference( other.getSpatialReference() );
-        for(int i=0;i<other.oMP.nGeomCount;i++)
+        OGRSurface::operator=(other);
+        for (const auto *poPoly : other.oMP)
         {
-            OGRTriangulatedSurface::addGeometry( other.oMP.getGeometryRef(i) );
+            OGRTriangulatedSurface::addGeometry(poPoly);
         }
     }
     return *this;
@@ -122,9 +79,9 @@ OGRTriangulatedSurface *OGRTriangulatedSurface::clone() const
  *
  */
 
-const char* OGRTriangulatedSurface::getGeometryName() const
+const char *OGRTriangulatedSurface::getGeometryName() const
 {
-    return "TIN" ;
+    return "TIN";
 }
 
 /************************************************************************/
@@ -138,11 +95,11 @@ const char* OGRTriangulatedSurface::getGeometryName() const
 
 OGRwkbGeometryType OGRTriangulatedSurface::getGeometryType() const
 {
-    if( (flags & OGR_G_3D) && (flags & OGR_G_MEASURED) )
+    if ((flags & OGR_G_3D) && (flags & OGR_G_MEASURED))
         return wkbTINZM;
-    else if( flags & OGR_G_MEASURED  )
+    else if (flags & OGR_G_MEASURED)
         return wkbTINM;
-    else if( flags & OGR_G_3D )
+    else if (flags & OGR_G_3D)
         return wkbTINZ;
     else
         return wkbTIN;
@@ -153,11 +110,12 @@ OGRwkbGeometryType OGRTriangulatedSurface::getGeometryType() const
 /************************************************************************/
 
 //! @cond Doxygen_Suppress
-OGRBoolean OGRTriangulatedSurface::isCompatibleSubType(
-                                        OGRwkbGeometryType eSubType ) const
+OGRBoolean
+OGRTriangulatedSurface::isCompatibleSubType(OGRwkbGeometryType eSubType) const
 {
-    return wkbFlatten( eSubType ) == wkbTriangle;
+    return wkbFlatten(eSubType) == wkbTriangle;
 }
+
 //! @endcond
 
 /************************************************************************/
@@ -165,10 +123,11 @@ OGRBoolean OGRTriangulatedSurface::isCompatibleSubType(
 /************************************************************************/
 
 //! @cond Doxygen_Suppress
-const char* OGRTriangulatedSurface::getSubGeometryName() const
+const char *OGRTriangulatedSurface::getSubGeometryName() const
 {
     return "TRIANGLE";
 }
+
 //! @endcond
 
 /************************************************************************/
@@ -180,25 +139,26 @@ OGRwkbGeometryType OGRTriangulatedSurface::getSubGeometryType() const
 {
     return wkbTriangle;
 }
+
 //! @endcond
 
 /************************************************************************/
 /*                            addGeometry()                             */
 /************************************************************************/
 
-OGRErr OGRTriangulatedSurface::addGeometry (const OGRGeometry *poNewGeom)
+OGRErr OGRTriangulatedSurface::addGeometry(const OGRGeometry *poNewGeom)
 {
     // If the geometry is a polygon, check if it can be cast as a triangle
-    if( EQUAL(poNewGeom->getGeometryName(),"POLYGON") )
+    if (EQUAL(poNewGeom->getGeometryName(), "POLYGON"))
     {
         OGRErr eErr = OGRERR_FAILURE;
-        OGRTriangle *poTriangle = new OGRTriangle(
-                    *(poNewGeom->toPolygon()), eErr);
+        OGRTriangle *poTriangle =
+            new OGRTriangle(*(poNewGeom->toPolygon()), eErr);
         if (poTriangle != nullptr && eErr == OGRERR_NONE)
         {
             eErr = addGeometryDirectly(poTriangle);
 
-            if( eErr != OGRERR_NONE )
+            if (eErr != OGRERR_NONE)
                 delete poTriangle;
 
             return eErr;
@@ -219,7 +179,7 @@ OGRErr OGRTriangulatedSurface::addGeometry (const OGRGeometry *poNewGeom)
 
 //! @cond Doxygen_Suppress
 OGRPolyhedralSurfaceCastToMultiPolygon
-                        OGRTriangulatedSurface::GetCasterToMultiPolygon() const
+OGRTriangulatedSurface::GetCasterToMultiPolygon() const
 {
     return OGRTriangulatedSurface::CastToMultiPolygonImpl;
 }
@@ -228,13 +188,13 @@ OGRPolyhedralSurfaceCastToMultiPolygon
 /*                         CastToMultiPolygon()                         */
 /************************************************************************/
 
-OGRMultiPolygon* OGRTriangulatedSurface::CastToMultiPolygonImpl(
-                                                    OGRPolyhedralSurface* poTS)
+OGRMultiPolygon *
+OGRTriangulatedSurface::CastToMultiPolygonImpl(OGRPolyhedralSurface *poTS)
 {
     OGRMultiPolygon *poMultiPolygon = new OGRMultiPolygon();
     poMultiPolygon->assignSpatialReference(poTS->getSpatialReference());
 
-    for( auto&& poSubGeom: *poTS )
+    for (auto &&poSubGeom : *poTS)
     {
         OGRPolygon *poPolygon = OGRSurface::CastToPolygon(poSubGeom);
         poMultiPolygon->addGeometryDirectly(poPolygon);
@@ -244,6 +204,7 @@ OGRMultiPolygon* OGRTriangulatedSurface::CastToMultiPolygonImpl(
 
     return poMultiPolygon;
 }
+
 //! @endcond
 
 /************************************************************************/
@@ -260,13 +221,12 @@ OGRMultiPolygon* OGRTriangulatedSurface::CastToMultiPolygonImpl(
  * @return new geometry.
  */
 
-
-OGRPolyhedralSurface* OGRTriangulatedSurface::CastToPolyhedralSurface(
-                                                OGRTriangulatedSurface* poTS)
+OGRPolyhedralSurface *
+OGRTriangulatedSurface::CastToPolyhedralSurface(OGRTriangulatedSurface *poTS)
 {
-    OGRPolyhedralSurface* poPS = new OGRPolyhedralSurface();
+    OGRPolyhedralSurface *poPS = new OGRPolyhedralSurface();
     poPS->assignSpatialReference(poTS->getSpatialReference());
-    for( auto&& poSubGeom: *poTS )
+    for (auto &&poSubGeom : *poTS)
     {
         OGRPolygon *poPolygon = OGRSurface::CastToPolygon(poSubGeom);
         poPS->oMP.addGeometryDirectly(poPolygon);
