@@ -110,13 +110,13 @@ static int CPLFindFilenameStart(const char *pszFilename, size_t nStart = 0)
  * filename.  If there is no path in the passed filename an empty string
  * will be returned (not NULL).
  *
- * <pre>
+ * \code{.cpp}
  * CPLGetPath( "abc/def.xyz" ) == "abc"
  * CPLGetPath( "/abc/def/" ) == "/abc/def"
  * CPLGetPath( "/" ) == "/"
  * CPLGetPath( "/abc/def" ) == "/abc"
  * CPLGetPath( "abc" ) == ""
- * </pre>
+ * \endcode
  *
  * @param pszFilename the filename potentially including a path.
  *
@@ -128,6 +128,10 @@ static int CPLFindFilenameStart(const char *pszFilename, size_t nStart = 0)
 const char *CPLGetPath(const char *pszFilename)
 
 {
+    char *pszStaticResult = CPLGetStaticResult();
+    if (pszStaticResult == nullptr)
+        return CPLStaticBufferTooSmall(pszStaticResult);
+
     size_t nSuffixPos = 0;
     if (STARTS_WITH(pszFilename, "/vsicurl/http"))
     {
@@ -135,11 +139,41 @@ const char *CPLGetPath(const char *pszFilename)
         if (pszQuestionMark)
             nSuffixPos = static_cast<size_t>(pszQuestionMark - pszFilename);
     }
+    else if (STARTS_WITH(pszFilename, "/vsicurl?") &&
+             strstr(pszFilename, "url="))
+    {
+        std::string osRet;
+        const CPLStringList aosTokens(
+            CSLTokenizeString2(pszFilename + strlen("/vsicurl?"), "&", 0));
+        for (int i = 0; i < aosTokens.size(); i++)
+        {
+            if (osRet.empty())
+                osRet = "/vsicurl?";
+            else
+                osRet += '&';
+            if (STARTS_WITH(aosTokens[i], "url=") &&
+                !STARTS_WITH(aosTokens[i], "url=/vsicurl"))
+            {
+                char *pszUnescaped =
+                    CPLUnescapeString(aosTokens[i], nullptr, CPLES_URL);
+                char *pszPath = CPLEscapeString(
+                    CPLGetPath(pszUnescaped + strlen("url=")), -1, CPLES_URL);
+                osRet += "url=";
+                osRet += pszPath;
+                CPLFree(pszPath);
+                CPLFree(pszUnescaped);
+            }
+            else
+            {
+                osRet += aosTokens[i];
+            }
+        }
+        CPLStrlcpy(pszStaticResult, osRet.c_str(), CPL_PATH_BUF_SIZE);
+        return pszStaticResult;
+    }
 
     const int iFileStart = CPLFindFilenameStart(pszFilename, nSuffixPos);
-    char *pszStaticResult = CPLGetStaticResult();
-
-    if (pszStaticResult == nullptr || iFileStart >= CPL_PATH_BUF_SIZE)
+    if (iFileStart >= CPL_PATH_BUF_SIZE)
         return CPLStaticBufferTooSmall(pszStaticResult);
 
     CPLAssert(!(pszFilename >= pszStaticResult &&
@@ -179,13 +213,13 @@ const char *CPLGetPath(const char *pszFilename)
  * filename.  If there is no path in the passed filename the dot will be
  * returned.  It is the only difference from CPLGetPath().
  *
- * <pre>
+ * \code{.cpp}
  * CPLGetDirname( "abc/def.xyz" ) == "abc"
  * CPLGetDirname( "/abc/def/" ) == "/abc/def"
  * CPLGetDirname( "/" ) == "/"
  * CPLGetDirname( "/abc/def" ) == "/abc"
  * CPLGetDirname( "abc" ) == "."
- * </pre>
+ * \endcode
  *
  * @param pszFilename the filename potentially including a path.
  *
@@ -197,6 +231,10 @@ const char *CPLGetPath(const char *pszFilename)
 const char *CPLGetDirname(const char *pszFilename)
 
 {
+    char *pszStaticResult = CPLGetStaticResult();
+    if (pszStaticResult == nullptr)
+        return CPLStaticBufferTooSmall(pszStaticResult);
+
     size_t nSuffixPos = 0;
     if (STARTS_WITH(pszFilename, "/vsicurl/http"))
     {
@@ -204,11 +242,42 @@ const char *CPLGetDirname(const char *pszFilename)
         if (pszQuestionMark)
             nSuffixPos = static_cast<size_t>(pszQuestionMark - pszFilename);
     }
+    else if (STARTS_WITH(pszFilename, "/vsicurl?") &&
+             strstr(pszFilename, "url="))
+    {
+        std::string osRet;
+        const CPLStringList aosTokens(
+            CSLTokenizeString2(pszFilename + strlen("/vsicurl?"), "&", 0));
+        for (int i = 0; i < aosTokens.size(); i++)
+        {
+            if (osRet.empty())
+                osRet = "/vsicurl?";
+            else
+                osRet += '&';
+            if (STARTS_WITH(aosTokens[i], "url=") &&
+                !STARTS_WITH(aosTokens[i], "url=/vsicurl"))
+            {
+                char *pszUnescaped =
+                    CPLUnescapeString(aosTokens[i], nullptr, CPLES_URL);
+                char *pszPath = CPLEscapeString(
+                    CPLGetDirname(pszUnescaped + strlen("url=")), -1,
+                    CPLES_URL);
+                osRet += "url=";
+                osRet += pszPath;
+                CPLFree(pszPath);
+                CPLFree(pszUnescaped);
+            }
+            else
+            {
+                osRet += aosTokens[i];
+            }
+        }
+        CPLStrlcpy(pszStaticResult, osRet.c_str(), CPL_PATH_BUF_SIZE);
+        return pszStaticResult;
+    }
 
     const int iFileStart = CPLFindFilenameStart(pszFilename, nSuffixPos);
-    char *pszStaticResult = CPLGetStaticResult();
-
-    if (pszStaticResult == nullptr || iFileStart >= CPL_PATH_BUF_SIZE)
+    if (iFileStart >= CPL_PATH_BUF_SIZE)
         return CPLStaticBufferTooSmall(pszStaticResult);
 
     CPLAssert(!(pszFilename >= pszStaticResult &&
@@ -248,11 +317,11 @@ const char *CPLGetDirname(const char *pszFilename)
  * filename.  If there is no filename (passed value ends in trailing directory
  * separator) an empty string is returned.
  *
- * <pre>
+ * \code{.cpp}
  * CPLGetFilename( "abc/def.xyz" ) == "def.xyz"
  * CPLGetFilename( "/abc/def/" ) == ""
  * CPLGetFilename( "abc/def" ) == "def"
- * </pre>
+ * \endcode
  *
  * @param pszFullFilename the full filename potentially including a path.
  *
@@ -279,11 +348,11 @@ const char *CPLGetFilename(const char *pszFullFilename)
  * name.  If there is no basename (passed value ends in trailing directory
  * separator, or filename starts with a dot) an empty string is returned.
  *
- * <pre>
+ * \code{.cpp}
  * CPLGetBasename( "abc/def.xyz" ) == "def"
  * CPLGetBasename( "abc/def" ) == "def"
  * CPLGetBasename( "abc/def/" ) == ""
- * </pre>
+ * \endcode
  *
  * @param pszFullFilename the full filename potentially including a path.
  *
@@ -334,10 +403,10 @@ const char *CPLGetBasename(const char *pszFullFilename)
  * name.  If there is no extension (the filename has no dot) an empty string
  * is returned.  The returned extension will not include the period.
  *
- * <pre>
+ * \code{.cpp}
  * CPLGetExtension( "abc/def.xyz" ) == "xyz"
  * CPLGetExtension( "abc/def" ) == ""
- * </pre>
+ * \endcode
  *
  * @param pszFullFilename the full filename potentially including a path.
  *
@@ -504,12 +573,13 @@ const char *CPLResetExtension(const char *pszPath, const char *pszExt)
  * The path, and extension are optional.  The basename may in fact contain
  * an extension if desired.
  *
- * <pre>
+ * \code{.cpp}
  * CPLFormFilename("abc/xyz", "def", ".dat" ) == "abc/xyz/def.dat"
  * CPLFormFilename(NULL,"def", NULL ) == "def"
  * CPLFormFilename(NULL, "abc/def.dat", NULL ) == "abc/def.dat"
  * CPLFormFilename("/abc/xyz/", "def.dat", NULL ) == "/abc/xyz/def.dat"
- * </pre>
+ * CPLFormFilename("/a/b/c", "../d", NULL ) == "/a/b/d" (since 3.10.1)
+ * \endcode
  *
  * @param pszPath directory path to the directory containing the file.  This
  * may be relative or absolute, and may have a trailing path separator or
@@ -562,33 +632,67 @@ const char *CPLFormFilename(const char *pszPath, const char *pszBasename,
         pszAddedPathSep = "/";
     }
 
-    if (!CPLIsFilenameRelative(pszPath) && strcmp(pszBasename, "..") == 0)
+    if (!CPLIsFilenameRelative(pszPath) && pszBasename[0] == '.' &&
+        pszBasename[1] == '.' &&
+        (pszBasename[2] == 0 || pszBasename[2] == '\\' ||
+         pszBasename[2] == '/'))
     {
-        // /a/b + .. --> /a
+        // "/a/b/" + "..[/something]" --> "/a[/something]"
+        // "/a/b" + "..[/something]" --> "/a[/something]"
         if (pszPath[nLenPath - 1] == '\\' || pszPath[nLenPath - 1] == '/')
             nLenPath--;
-        size_t nLenPathOri = nLenPath;
-        while (nLenPath > 0 && pszPath[nLenPath - 1] != '\\' &&
-               pszPath[nLenPath - 1] != '/')
+        while (true)
         {
-            nLenPath--;
-        }
-        if (nLenPath == 1 && pszPath[0] == '/')
-        {
-            pszBasename = "";
-        }
-        else if ((nLenPath > 1 && pszPath[0] == '/') ||
-                 (nLenPath > 2 && pszPath[1] == ':') ||
-                 (nLenPath > 6 && strncmp(pszPath, "\\\\$\\", 4) == 0))
-        {
-            nLenPath--;
-            pszBasename = "";
-        }
-        else
-        {
-            nLenPath = nLenPathOri;
-            if (pszAddedPathSep[0] == 0)
-                pszAddedPathSep = VSIGetDirectorySeparator(pszPath);
+            const char *pszBasenameOri = pszBasename;
+            const size_t nLenPathOri = nLenPath;
+            while (nLenPath > 0 && pszPath[nLenPath - 1] != '\\' &&
+                   pszPath[nLenPath - 1] != '/')
+            {
+                nLenPath--;
+            }
+            if (nLenPath == 1 && pszPath[0] == '/')
+            {
+                pszBasename += 2;
+                if (pszBasename[0] == '/' || pszBasename[0] == '\\')
+                    pszBasename++;
+                if (*pszBasename == '.')
+                {
+                    pszBasename = pszBasenameOri;
+                    nLenPath = nLenPathOri;
+                    if (pszAddedPathSep[0] == 0)
+                        pszAddedPathSep =
+                            pszPath[0] == '/'
+                                ? "/"
+                                : VSIGetDirectorySeparator(pszPath);
+                }
+                break;
+            }
+            else if ((nLenPath > 1 && pszPath[0] == '/') ||
+                     (nLenPath > 2 && pszPath[1] == ':') ||
+                     (nLenPath > 6 && strncmp(pszPath, "\\\\$\\", 4) == 0))
+            {
+                nLenPath--;
+                pszBasename += 2;
+                if ((pszBasename[0] == '/' || pszBasename[0] == '\\') &&
+                    pszBasename[1] == '.' && pszBasename[2] == '.')
+                {
+                    pszBasename++;
+                }
+                else
+                {
+                    break;
+                }
+            }
+            else
+            {
+                pszBasename = pszBasenameOri;
+                nLenPath = nLenPathOri;
+                if (pszAddedPathSep[0] == 0)
+                    pszAddedPathSep = pszPath[0] == '/'
+                                          ? "/"
+                                          : VSIGetDirectorySeparator(pszPath);
+                break;
+            }
         }
     }
     else if (nLenPath > 0 && pszPath[nLenPath - 1] != '/' &&
@@ -695,8 +799,12 @@ const char *CPLFormCIFilename(const char *pszPath, const char *pszBasename,
             pszFilename[i] = static_cast<char>(CPLToupper(pszFilename[i]));
         }
 
-        pszFullPath = CPLFormFilename(pszPath, pszFilename, nullptr);
-        nStatRet = VSIStatExL(pszFullPath, &sStatBuf, VSI_STAT_EXISTS_FLAG);
+        const std::string osTmpPath(
+            CPLFormFilename(pszPath, pszFilename, nullptr));
+        nStatRet =
+            VSIStatExL(osTmpPath.c_str(), &sStatBuf, VSI_STAT_EXISTS_FLAG);
+        if (nStatRet == 0)
+            pszFullPath = CPLFormFilename(pszPath, pszFilename, nullptr);
     }
 
     if (nStatRet != 0)
@@ -707,8 +815,12 @@ const char *CPLFormCIFilename(const char *pszPath, const char *pszBasename,
                 CPLTolower(static_cast<unsigned char>(pszFilename[i])));
         }
 
-        pszFullPath = CPLFormFilename(pszPath, pszFilename, nullptr);
-        nStatRet = VSIStatExL(pszFullPath, &sStatBuf, VSI_STAT_EXISTS_FLAG);
+        const std::string osTmpPath(
+            CPLFormFilename(pszPath, pszFilename, nullptr));
+        nStatRet =
+            VSIStatExL(osTmpPath.c_str(), &sStatBuf, VSI_STAT_EXISTS_FLAG);
+        if (nStatRet == 0)
+            pszFullPath = CPLFormFilename(pszPath, pszFilename, nullptr);
     }
 
     if (nStatRet != 0)
@@ -732,13 +844,13 @@ const char *CPLFormCIFilename(const char *pszPath, const char *pszBasename,
  * absolute, rather than relative, then it will be returned unaltered.
  *
  * Examples:
- * <pre>
+ * \code{.cpp}
  * CPLProjectRelativeFilename("abc/def", "tmp/abc.gif") == "abc/def/tmp/abc.gif"
  * CPLProjectRelativeFilename("abc/def", "/tmp/abc.gif") == "/tmp/abc.gif"
  * CPLProjectRelativeFilename("/xy", "abc.gif") == "/xy/abc.gif"
  * CPLProjectRelativeFilename("/abc/def", "../abc.gif") == "/abc/def/../abc.gif"
  * CPLProjectRelativeFilename("C:\WIN", "abc.gif") == "C:\WIN\abc.gif"
- * </pre>
+ * \endcode
  *
  * @param pszProjectDir the directory relative to which the secondary files
  * path should be interpreted.
@@ -923,13 +1035,13 @@ const char *CPLExtractRelativePath(const char *pszBaseDir,
  * trailing slash removed. If there is no path in the passed filename
  * an empty string will be returned (not NULL).
  *
- * <pre>
+ * \code{.cpp}
  * CPLCleanTrailingSlash( "abc/def/" ) == "abc/def"
  * CPLCleanTrailingSlash( "abc/def" ) == "abc/def"
  * CPLCleanTrailingSlash( "c:\\abc\\def\\" ) == "c:\\abc\\def"
  * CPLCleanTrailingSlash( "c:\\abc\\def" ) == "c:\\abc\\def"
  * CPLCleanTrailingSlash( "abc" ) == "abc"
- * </pre>
+ * \endcode
  *
  * @param pszPath the path to be cleaned up
  *
