@@ -10,23 +10,7 @@
  * Copyright (c) 1999-2008, Frank Warmerdam
  * Copyright (c) 2008-2014, Even Rouault <even dot rouault at spatialys.com>
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ******************************************************************************
  */
 
@@ -924,6 +908,78 @@ void **CPLQuadTreeSearch(const CPLQuadTree *hQuadTree, const CPLRectObj *pAoi,
     return ppFeatureList;
 }
 
+/************************************************************************/
+/*                         CPLQuadTreeHasMatch()                        */
+/************************************************************************/
+
+static bool CPLQuadTreeHasMatch(const CPLQuadTree *hQuadTree,
+                                const QuadTreeNode *psNode,
+                                const CPLRectObj *pAoi)
+{
+    /* -------------------------------------------------------------------- */
+    /*      Does this psNode overlap the area of interest at all?           */
+    /* -------------------------------------------------------------------- */
+    if (!CPL_RectOverlap(&psNode->rect, pAoi))
+        return false;
+
+    /* -------------------------------------------------------------------- */
+    /*      Check the local features.                                       */
+    /* -------------------------------------------------------------------- */
+    for (int i = 0; i < psNode->nFeatures; i++)
+    {
+        if (hQuadTree->pfnGetBounds == nullptr &&
+            hQuadTree->pfnGetBoundsEx == nullptr)
+        {
+            if (CPL_RectOverlap(&psNode->pasBounds[i], pAoi))
+                return true;
+        }
+        else
+        {
+            CPLRectObj bounds;
+            if (hQuadTree->pfnGetBoundsEx)
+                hQuadTree->pfnGetBoundsEx(psNode->pahFeatures[i],
+                                          hQuadTree->pUserData, &bounds);
+            else
+                hQuadTree->pfnGetBounds(psNode->pahFeatures[i], &bounds);
+
+            if (CPL_RectOverlap(&bounds, pAoi))
+                return true;
+        }
+    }
+
+    /* -------------------------------------------------------------------- */
+    /*      Recurse to subnodes if they exist.                              */
+    /* -------------------------------------------------------------------- */
+    for (int i = 0; i < psNode->nNumSubNodes; i++)
+    {
+        if (psNode->apSubNode[i])
+        {
+            if (CPLQuadTreeHasMatch(hQuadTree, psNode->apSubNode[i], pAoi))
+            {
+                return true;
+            }
+        }
+    }
+
+    return false;
+}
+
+/**
+ * Returns whether the quadtree has at least one element whose bounding box
+ * intersects the provided area of interest
+ *
+ * @param hQuadTree the quad tree
+ * @param pAoi the pointer to the area of interest
+ */
+
+bool CPLQuadTreeHasMatch(const CPLQuadTree *hQuadTree, const CPLRectObj *pAoi)
+{
+    CPLAssert(hQuadTree);
+    CPLAssert(pAoi);
+
+    return CPLQuadTreeHasMatch(hQuadTree, hQuadTree->psRoot, pAoi);
+}
+
 /************************************************************************/
 /*                    CPLQuadTreeNodeForeach()                          */
 /************************************************************************/
