@@ -8,23 +8,7 @@
  * Copyright (c) 2011, Frank Warmerdam <warmerdam@pobox.com>
  * Copyright (c) 2011-2014, Even Rouault <even dot rouault at spatialys.com>
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #include "cpl_port.h"
@@ -64,6 +48,7 @@ class VSICachedFile final : public VSIVirtualHandle
   public:
     VSICachedFile(VSIVirtualHandle *poBaseHandle, size_t nChunkSize,
                   size_t nCacheSize);
+
     ~VSICachedFile() override
     {
         VSICachedFile::Close();
@@ -82,6 +67,7 @@ class VSICachedFile final : public VSIVirtualHandle
         m_oCache;  // can only been initialized in constructor
 
     bool m_bEOF = false;
+    bool m_bError = false;
 
     int Seek(vsi_l_offset nOffset, int nWhence) override;
     vsi_l_offset Tell() override;
@@ -97,9 +83,12 @@ class VSICachedFile final : public VSIVirtualHandle
     }
 
     size_t Write(const void *pBuffer, size_t nSize, size_t nMemb) override;
+    void ClearErr() override;
     int Eof() override;
+    int Error() override;
     int Flush() override;
     int Close() override;
+
     void *GetNativeFileDescriptor() override
     {
         return m_poBase->GetNativeFileDescriptor();
@@ -109,6 +98,7 @@ class VSICachedFile final : public VSIVirtualHandle
     {
         return m_poBase->HasPRead();
     }
+
     size_t PRead(void *pBuffer, size_t nSize,
                  vsi_l_offset nOffset) const override
     {
@@ -146,7 +136,8 @@ template <class T> inline T DIV_ROUND_UP(T a, T b)
 
 VSICachedFile::VSICachedFile(VSIVirtualHandle *poBaseHandle, size_t nChunkSize,
                              size_t nCacheSize)
-    : m_poBase(poBaseHandle), m_nChunkSize(nChunkSize ? nChunkSize : 32768),
+    : m_poBase(poBaseHandle),
+      m_nChunkSize(nChunkSize ? nChunkSize : VSI_CACHED_DEFAULT_CHUNK_SIZE),
       m_oCache{DIV_ROUND_UP(GetCacheMax(nCacheSize), m_nChunkSize), 0}
 {
     m_poBase->Seek(0, SEEK_END);
@@ -237,6 +228,8 @@ bool VSICachedFile::LoadBlocks(vsi_l_offset nStartBlock, size_t nBlockCount,
                 m_poBase->Read(oData.data(), 1, m_nChunkSize);
             if (nDataRead == 0)
                 return false;
+            if (nDataRead < m_nChunkSize && m_poBase->Error())
+                m_bError = true;
             oData.resize(nDataRead);
 
             m_oCache.insert(nStartBlock, std::move(oData));
@@ -288,11 +281,13 @@ bool VSICachedFile::LoadBlocks(vsi_l_offset nStartBlock, size_t nBlockCount,
     /*      Read the whole request into the working buffer.                 */
     /* -------------------------------------------------------------------- */
 
-    const size_t nDataRead =
-        m_poBase->Read(pabyWorkBuffer, 1, nBlockCount * m_nChunkSize);
+    const size_t nToRead = nBlockCount * m_nChunkSize;
+    const size_t nDataRead = m_poBase->Read(pabyWorkBuffer, 1, nToRead);
+    if (nDataRead < nToRead && m_poBase->Error())
+        m_bError = true;
 
     bool ret = true;
-    if (nBlockCount * m_nChunkSize > nDataRead + m_nChunkSize - 1)
+    if (nToRead > nDataRead + m_nChunkSize - 1)
     {
         size_t nNewBlockCount = (nDataRead + m_nChunkSize - 1) / m_nChunkSize;
         if (nNewBlockCount < nBlockCount)
@@ -356,8 +351,15 @@ size_t VSICachedFile::Read(void *pBuffer, size_t nSize, size_t nCount)
     /*      Make sure the cache is loaded for the whole request region.     */
     /* ==================================================================== */
     const vsi_l_offset nStartBlock = m_nOffset / m_nChunkSize;
-    const vsi_l_offset nEndBlock =
-        (m_nOffset + nRequestedBytes - 1) / m_nChunkSize;
+    // Calculate last block
+    const vsi_l_offset nLastBlock = m_nFileSize / m_nChunkSize;
+    vsi_l_offset nEndBlock = (m_nOffset + nRequestedBytes - 1) / m_nChunkSize;
+
+    // if nLastBlock is not 0 consider the min value to avoid out-of-range reads
+    if (nLastBlock != 0 && nEndBlock > nLastBlock)
+    {
+        nEndBlock = nLastBlock;
+    }
 
     for (vsi_l_offset iBlock = nStartBlock; iBlock <= nEndBlock; iBlock++)
     {
@@ -418,7 +420,7 @@ size_t VSICachedFile::Read(void *pBuffer, size_t nSize, size_t nCount)
     m_nOffset += nAmountCopied;
 
     const size_t nRet = nAmountCopied / nSize;
-    if (nRet != nCount)
+    if (nRet != nCount && !m_bError)
         m_bEOF = true;
     return nRet;
 }
@@ -445,6 +447,28 @@ size_t VSICachedFile::Write(const void * /* pBuffer */, size_t /*nSize */,
     return 0;
 }
 
+/************************************************************************/
+/*                             ClearErr()                               */
+/************************************************************************/
+
+void VSICachedFile::ClearErr()
+
+{
+    m_poBase->ClearErr();
+    m_bEOF = false;
+    m_bError = false;
+}
+
+/************************************************************************/
+/*                              Error()                                 */
+/************************************************************************/
+
+int VSICachedFile::Error()
+
+{
+    return m_bError;
+}
+
 /************************************************************************/
 /*                                Eof()                                 */
 /************************************************************************/
@@ -465,15 +489,252 @@ int VSICachedFile::Flush()
     return 0;
 }
 
+/************************************************************************/
+/*                      VSICachedFilesystemHandler                      */
+/************************************************************************/
+
+class VSICachedFilesystemHandler final : public VSIFilesystemHandler
+{
+    static bool AnalyzeFilename(const char *pszFilename,
+                                std::string &osUnderlyingFilename,
+                                size_t &nChunkSize, size_t &nCacheSize);
+
+  public:
+    VSIVirtualHandle *Open(const char *pszFilename, const char *pszAccess,
+                           bool bSetError, CSLConstList papszOptions) override;
+    int Stat(const char *pszFilename, VSIStatBufL *pStatBuf,
+             int nFlags) override;
+    char **ReadDirEx(const char *pszDirname, int nMaxFiles) override;
+};
+
+/************************************************************************/
+/*                               ParseSize()                            */
+/************************************************************************/
+
+static bool ParseSize(const char *pszKey, const char *pszValue, size_t nMaxVal,
+                      size_t &nOutVal)
+{
+    char *end = nullptr;
+    auto nVal = std::strtoull(pszValue, &end, 10);
+    if (!end || end == pszValue || nVal >= nMaxVal)
+    {
+        CPLError(
+            CE_Failure, CPLE_IllegalArg,
+            "Invalid value for %s: %s. Max supported value = " CPL_FRMT_GUIB,
+            pszKey, pszValue, static_cast<GUIntBig>(nMaxVal));
+        return false;
+    }
+    if (*end != '\0')
+    {
+        if (strcmp(end, "KB") == 0)
+        {
+            if (nVal > nMaxVal / 1024)
+            {
+                CPLError(CE_Failure, CPLE_IllegalArg,
+                         "Invalid value for %s: %s. Max supported value "
+                         "= " CPL_FRMT_GUIB,
+                         pszKey, pszValue, static_cast<GUIntBig>(nMaxVal));
+                return false;
+            }
+            nVal *= 1024;
+        }
+        else if (strcmp(end, "MB") == 0)
+        {
+            if (nVal > nMaxVal / (1024 * 1024))
+            {
+                CPLError(CE_Failure, CPLE_IllegalArg,
+                         "Invalid value for %s: %s. Max supported value "
+                         "= " CPL_FRMT_GUIB,
+                         pszKey, pszValue, static_cast<GUIntBig>(nMaxVal));
+                return false;
+            }
+            nVal *= (1024 * 1024);
+        }
+        else
+        {
+            CPLError(CE_Failure, CPLE_IllegalArg, "Invalid value for %s: %s",
+                     pszKey, pszValue);
+            return false;
+        }
+    }
+    nOutVal = static_cast<size_t>(nVal);
+    return true;
+}
+
+/************************************************************************/
+/*                          AnalyzeFilename()                           */
+/************************************************************************/
+
+bool VSICachedFilesystemHandler::AnalyzeFilename(
+    const char *pszFilename, std::string &osUnderlyingFilename,
+    size_t &nChunkSize, size_t &nCacheSize)
+{
+
+    if (!STARTS_WITH(pszFilename, "/vsicached?"))
+        return false;
+
+    const CPLStringList aosTokens(
+        CSLTokenizeString2(pszFilename + strlen("/vsicached?"), "&", 0));
+
+    osUnderlyingFilename.clear();
+    nChunkSize = 0;
+    nCacheSize = 0;
+
+    for (int i = 0; i < aosTokens.size(); ++i)
+    {
+        char *pszUnescaped =
+            CPLUnescapeString(aosTokens[i], nullptr, CPLES_URL);
+        std::string osUnescaped(pszUnescaped);
+        CPLFree(pszUnescaped);
+        char *pszKey = nullptr;
+        const char *pszValue = CPLParseNameValue(osUnescaped.c_str(), &pszKey);
+        if (pszKey && pszValue)
+        {
+            if (strcmp(pszKey, "file") == 0)
+            {
+                osUnderlyingFilename = pszValue;
+            }
+            else if (strcmp(pszKey, "chunk_size") == 0)
+            {
+                if (!ParseSize(pszKey, pszValue, 1024 * 1024 * 1024,
+                               nChunkSize))
+                {
+                    CPLFree(pszKey);
+                    return false;
+                }
+            }
+            else if (strcmp(pszKey, "cache_size") == 0)
+            {
+                if (!ParseSize(pszKey, pszValue,
+                               std::numeric_limits<size_t>::max(), nCacheSize))
+                {
+                    CPLFree(pszKey);
+                    return false;
+                }
+            }
+            else
+            {
+                CPLError(CE_Warning, CPLE_NotSupported,
+                         "Unsupported option: %s", pszKey);
+            }
+        }
+        CPLFree(pszKey);
+    }
+
+    if (osUnderlyingFilename.empty())
+    {
+        CPLError(CE_Warning, CPLE_NotSupported, "Missing 'file' option");
+    }
+
+    return !osUnderlyingFilename.empty();
+}
+
+/************************************************************************/
+/*                               Open()                                 */
+/************************************************************************/
+
+VSIVirtualHandle *VSICachedFilesystemHandler::Open(const char *pszFilename,
+                                                   const char *pszAccess,
+                                                   bool bSetError,
+                                                   CSLConstList papszOptions)
+{
+    std::string osUnderlyingFilename;
+    size_t nChunkSize = 0;
+    size_t nCacheSize = 0;
+    if (!AnalyzeFilename(pszFilename, osUnderlyingFilename, nChunkSize,
+                         nCacheSize))
+        return nullptr;
+    if (strcmp(pszAccess, "r") != 0 && strcmp(pszAccess, "rb") != 0)
+    {
+        if (bSetError)
+        {
+            VSIError(VSIE_FileError,
+                     "/vsicached? supports only 'r' and 'rb' access modes");
+        }
+        return nullptr;
+    }
+
+    auto fp = VSIFOpenEx2L(osUnderlyingFilename.c_str(), pszAccess, bSetError,
+                           papszOptions);
+    if (!fp)
+        return nullptr;
+    return VSICreateCachedFile(fp, nChunkSize, nCacheSize);
+}
+
+/************************************************************************/
+/*                               Stat()                                 */
+/************************************************************************/
+
+int VSICachedFilesystemHandler::Stat(const char *pszFilename,
+                                     VSIStatBufL *pStatBuf, int nFlags)
+{
+    std::string osUnderlyingFilename;
+    size_t nChunkSize = 0;
+    size_t nCacheSize = 0;
+    if (!AnalyzeFilename(pszFilename, osUnderlyingFilename, nChunkSize,
+                         nCacheSize))
+        return -1;
+    return VSIStatExL(osUnderlyingFilename.c_str(), pStatBuf, nFlags);
+}
+
+/************************************************************************/
+/*                          ReadDirEx()                                 */
+/************************************************************************/
+
+char **VSICachedFilesystemHandler::ReadDirEx(const char *pszDirname,
+                                             int nMaxFiles)
+{
+    std::string osUnderlyingFilename;
+    size_t nChunkSize = 0;
+    size_t nCacheSize = 0;
+    if (!AnalyzeFilename(pszDirname, osUnderlyingFilename, nChunkSize,
+                         nCacheSize))
+        return nullptr;
+    return VSIReadDirEx(osUnderlyingFilename.c_str(), nMaxFiles);
+}
+
 //! @endcond
 
 /************************************************************************/
 /*                        VSICreateCachedFile()                         */
 /************************************************************************/
 
+/** Wraps a file handle in another one, which has caching for read-operations.
+ *
+ * This takes a virtual file handle and returns a new handle that caches
+ * read-operations on the input file handle. The cache is RAM based and
+ * the content of the cache is discarded when the file handle is closed.
+ * The cache is a least-recently used lists of blocks of 32KB each.
+ *
+ * @param poBaseHandle base handle
+ * @param nChunkSize chunk size, in bytes. If 0, defaults to 32 KB
+ * @param nCacheSize total size of the cache for the file, in bytes.
+ *                   If 0, defaults to the value of the VSI_CACHE_SIZE
+ *                   configuration option, which defaults to 25 MB.
+ * @return a new handle
+ */
 VSIVirtualHandle *VSICreateCachedFile(VSIVirtualHandle *poBaseHandle,
                                       size_t nChunkSize, size_t nCacheSize)
 
 {
     return new VSICachedFile(poBaseHandle, nChunkSize, nCacheSize);
 }
+
+/************************************************************************/
+/*                   VSIInstallCachedFileHandler()                      */
+/************************************************************************/
+
+/*!
+ \brief Install /vsicached? file system handler
+
+ \verbatim embed:rst
+ See :ref:`/vsicached? documentation <vsicached>`
+ \endverbatim
+
+ @since GDAL 3.8.0
+ */
+void VSIInstallCachedFileHandler(void)
+{
+    VSIFilesystemHandler *poHandler = new VSICachedFilesystemHandler;
+    VSIFileManager::InstallHandler("/vsicached?", poHandler);
+}
