@@ -8,23 +8,7 @@
  * Copyright (c) 1998, Frank Warmerdam
  * Copyright (c) 2008-2012, Even Rouault <even dot rouault at spatialys.com>
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************
  *
  * NB: Note that in wrappers we are always saving the error state (errno
@@ -56,6 +40,7 @@
 #include <cstdlib>
 #include <cstring>
 #include <ctime>
+#include <limits>
 #if HAVE_SYS_STAT_H
 #include <sys/stat.h>
 #endif
@@ -105,7 +90,7 @@ extern "C" GIntBig CPL_DLL CPL_STDCALL GDALGetCacheUsed64(void);
 #endif
 
 /* Unix or Windows NT/2000/XP */
-#if !defined(WIN32)
+#if !defined(_WIN32)
 #include <unistd.h>
 #else
 #include <io.h>
@@ -120,7 +105,7 @@ extern "C" GIntBig CPL_DLL CPL_STDCALL GDALGetCacheUsed64(void);
 FILE *VSIFOpen(const char *pszFilename, const char *pszAccess)
 
 {
-#if defined(WIN32)
+#if defined(_WIN32)
     FILE *fp = nullptr;
     if (CPLTestBool(CPLGetConfigOption("GDAL_FILENAME_IS_UTF8", "YES")))
     {
@@ -907,6 +892,11 @@ void VSIFree(void *pData)
 
 void *VSIMallocAligned(size_t nAlignment, size_t nSize)
 {
+    // In particular for posix_memalign() where behavior when passing
+    // nSize == 0 is technically implementation defined (Valgrind complains),
+    // so let's always return NULL.
+    if (nSize == 0)
+        return nullptr;
 #if defined(HAVE_POSIX_MEMALIGN) && !defined(DEBUG_VSIMALLOC)
     void *pRet = nullptr;
     if (posix_memalign(&pRet, nAlignment, nSize) != 0)
@@ -924,7 +914,6 @@ void *VSIMallocAligned(size_t nAlignment, size_t nSize)
     // Detect overflow.
     if (nSize + nAlignment < nSize)
         return nullptr;
-    // TODO(schwehr): C++11 has std::aligned_storage, alignas, and related.
     GByte *pabyData = static_cast<GByte *>(VSIMalloc(nSize + nAlignment));
     if (pabyData == nullptr)
         return nullptr;
@@ -1205,6 +1194,7 @@ void *VSIMalloc3Verbose(size_t nSize1, size_t nSize2, size_t nSize3,
     }
     return pRet;
 }
+
 /************************************************************************/
 /*                          VSICallocVerbose()                          */
 /************************************************************************/
@@ -1266,7 +1256,7 @@ char *VSIStrdupVerbose(const char *pszStr, const char *pszFile, int nLine)
 int VSIStat(const char *pszFilename, VSIStatBuf *pStatBuf)
 
 {
-#if defined(WIN32)
+#if defined(_WIN32)
     if (CPLTestBool(CPLGetConfigOption("GDAL_FILENAME_IS_UTF8", "YES")))
     {
         wchar_t *pwszFilename =
@@ -1396,8 +1386,11 @@ GIntBig CPLGetPhysicalRAM(void)
 {
     const long nPhysPages = sysconf(_SC_PHYS_PAGES);
     const long nPageSize = sysconf(_SC_PAGESIZE);
-    if (nPhysPages < 0 || nPageSize < 0)
+    if (nPhysPages <= 0 || nPageSize <= 0 ||
+        nPhysPages > std::numeric_limits<GIntBig>::max() / nPageSize)
+    {
         return 0;
+    }
     GIntBig nVal = static_cast<GIntBig>(nPhysPages) * nPageSize;
 
 #ifdef __linux
@@ -1406,32 +1399,35 @@ GIntBig CPLGetPhysicalRAM(void)
         // which seems to be necessary for some container solutions
         // Cf https://lists.osgeo.org/pipermail/gdal-dev/2023-January/056784.html
         FILE *f = fopen("/proc/meminfo", "rb");
-        char szLine[256];
-        while (fgets(szLine, sizeof(szLine), f))
+        if (f)
         {
-            // Find line like "MemTotal:       32525176 kB"
-            if (strncmp(szLine, "MemTotal:", strlen("MemTotal:")) == 0)
+            char szLine[256];
+            while (fgets(szLine, sizeof(szLine), f))
             {
-                char *pszVal = szLine + strlen("MemTotal:");
-                pszVal += strspn(pszVal, " ");
-                char *pszEnd = strstr(pszVal, " kB");
-                if (pszEnd)
+                // Find line like "MemTotal:       32525176 kB"
+                if (strncmp(szLine, "MemTotal:", strlen("MemTotal:")) == 0)
                 {
-                    *pszEnd = 0;
-                    if (CPLGetValueType(pszVal) == CPL_VALUE_INTEGER)
+                    char *pszVal = szLine + strlen("MemTotal:");
+                    pszVal += strspn(pszVal, " ");
+                    char *pszEnd = strstr(pszVal, " kB");
+                    if (pszEnd)
                     {
-                        const GUIntBig nLimit =
-                            CPLScanUIntBig(pszVal,
-                                           static_cast<int>(strlen(pszVal))) *
-                            1024;
-                        nVal = static_cast<GIntBig>(
-                            std::min(static_cast<GUIntBig>(nVal), nLimit));
+                        *pszEnd = 0;
+                        if (CPLGetValueType(pszVal) == CPL_VALUE_INTEGER)
+                        {
+                            const GUIntBig nLimit =
+                                CPLScanUIntBig(
+                                    pszVal, static_cast<int>(strlen(pszVal))) *
+                                1024;
+                            nVal = static_cast<GIntBig>(
+                                std::min(static_cast<GUIntBig>(nVal), nLimit));
+                        }
                     }
+                    break;
                 }
-                break;
             }
+            fclose(f);
         }
-        fclose(f);
     }
 
     char szGroupName[256];
@@ -1439,34 +1435,37 @@ GIntBig CPLGetPhysicalRAM(void)
     szGroupName[0] = 0;
     {
         FILE *f = fopen("/proc/self/cgroup", "rb");
-        char szLine[256];
-        // Find line like "6:memory:/user.slice/user-1000.slice/user@1000.service"
-        // and store "/user.slice/user-1000.slice/user@1000.service" in
-        // szMemoryPath for cgroup V1 or single line "0::/...." for cgroup V2.
-        while (fgets(szLine, sizeof(szLine), f))
+        if (f)
         {
-            const char *pszMemory = strstr(szLine, ":memory:");
-            if (pszMemory)
+            char szLine[256];
+            // Find line like "6:memory:/user.slice/user-1000.slice/user@1000.service"
+            // and store "/user.slice/user-1000.slice/user@1000.service" in
+            // szMemoryPath for cgroup V1 or single line "0::/...." for cgroup V2.
+            while (fgets(szLine, sizeof(szLine), f))
             {
-                bFromMemory = true;
-                snprintf(szGroupName, sizeof(szGroupName), "%s",
-                         pszMemory + strlen(":memory:"));
-                char *pszEOL = strchr(szGroupName, '\n');
-                if (pszEOL)
-                    *pszEOL = '\0';
-                break;
-            }
-            if (strncmp(szLine, "0::", strlen("0::")) == 0)
-            {
-                snprintf(szGroupName, sizeof(szGroupName), "%s",
-                         szLine + strlen("0::"));
-                char *pszEOL = strchr(szGroupName, '\n');
-                if (pszEOL)
-                    *pszEOL = '\0';
-                break;
+                const char *pszMemory = strstr(szLine, ":memory:");
+                if (pszMemory)
+                {
+                    bFromMemory = true;
+                    snprintf(szGroupName, sizeof(szGroupName), "%s",
+                             pszMemory + strlen(":memory:"));
+                    char *pszEOL = strchr(szGroupName, '\n');
+                    if (pszEOL)
+                        *pszEOL = '\0';
+                    break;
+                }
+                if (strncmp(szLine, "0::", strlen("0::")) == 0)
+                {
+                    snprintf(szGroupName, sizeof(szGroupName), "%s",
+                             szLine + strlen("0::"));
+                    char *pszEOL = strchr(szGroupName, '\n');
+                    if (pszEOL)
+                        *pszEOL = '\0';
+                    break;
+                }
             }
+            fclose(f);
         }
-        fclose(f);
     }
     if (szGroupName[0])
     {
@@ -1559,7 +1558,7 @@ GIntBig CPLGetPhysicalRAM(void)
     return nPhysMem;
 }
 
-#elif defined(WIN32)
+#elif defined(_WIN32)
 
 // GlobalMemoryStatusEx requires _WIN32_WINNT >= 0x0500.
 #ifndef _WIN32_WINNT
