@@ -7,23 +7,7 @@
  ******************************************************************************
  * Copyright (c) 2012-2014, Even Rouault <even dot rouault at spatialys.com>
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #include "cpl_port.h"
@@ -766,10 +750,10 @@ static xmlParserInputPtr CPLExternalEntityLoader(const char *URL,
         }
         else if (strcmp(URL, "http://www.w3.org/2001/xml.xsd") == 0)
         {
-            CPLString osTmp = CPLFindLocalXSD("xml.xsd");
+            std::string osTmp = CPLFindLocalXSD("xml.xsd");
             if (!osTmp.empty())
             {
-                osURL = osTmp;
+                osURL = std::move(osTmp);
                 URL = osURL.c_str();
             }
             else
@@ -782,10 +766,10 @@ static xmlParserInputPtr CPLExternalEntityLoader(const char *URL,
         }
         else if (strcmp(URL, "http://www.w3.org/1999/xlink.xsd") == 0)
         {
-            CPLString osTmp = CPLFindLocalXSD("xlink.xsd");
+            std::string osTmp = CPLFindLocalXSD("xlink.xsd");
             if (!osTmp.empty())
             {
-                osURL = osTmp;
+                osURL = std::move(osTmp);
                 URL = osURL.c_str();
             }
             else
